searchData={"items":[{"type":"module","title":"Xandra","doc":"This module provides the main API to interface with Cassandra.\n\nThis module handles the connection to Cassandra, queries, connection pooling,\nconnection backoff, logging, and more.\n\n> #### `db_connection` {: .warning}\n>\n> Before v0.18.0, this library was built on top of the\n> [`db_connection`](http://hex.pm/packages/db_connection) library. Since v0.18.0,\n> this is not the case anymore. This allowed us to significantly improve the design\n> and architecture of Xandra, ultimately resulting in a client that is more fitted\n> for the Cassandra protocol.","ref":"Xandra.html"},{"type":"module","title":"Errors - Xandra","doc":"Many of the functions in this module (whose names don't end with a `!`)\nreturn values in the form `{:ok, result}` or `{:error, error}`. While `result`\nvaries based on the specific function, `error` is always one of the following:\n\n  * a `Xandra.Error` struct: such structs represent errors returned by\n    Cassandra. When such an error is returned, it means that communicating\n    with the Cassandra server was successful, but the server returned an\n    error. Examples of these errors are syntax errors in queries, non-existent\n    tables, and so on. See `Xandra.Error` for more information.\n\n  * a `Xandra.ConnectionError` struct: such structs represent errors in the\n    communication with the Cassandra server. For example, if the Cassandra\n    server dies while the connection is waiting for a response from the\n    server, a `Xandra.ConnectionError` error will be returned. See\n    `Xandra.ConnectionError` for more information.","ref":"Xandra.html#module-errors"},{"type":"module","title":"Parameters, encoding, and types - Xandra","doc":"Xandra supports parameterized queries (queries that specify \"parameter\" values\nthrough `?` or `:named_value`):\n\n    SELECT * FROM users WHERE name = ? AND email = ?\n    SELECT * FROM users WHERE name = :name AND email = :email\n\nWhen a query has positional parameters, parameters can be passed as a list to\nfunctions like `execute/4`: in this case, a parameter in a given position in\nthe list will be used as the `?` in the corresponding position in the\nquery. When a query has named parameters, parameters are passed as a map with\nstring keys representing each parameter's name and values representing the\ncorresponding parameter's value.\n\n#","ref":"Xandra.html#module-parameters-encoding-and-types"},{"type":"module","title":"Types - Xandra","doc":"For information about how Elixir types translate to Cassandra types and\nviceversa, see the [\"Data types comparison table\" page](data-types-comparison-table.html).\n\nCassandra supports many types of values, and some types have \"shades\" that\ncannot be represented by Elixir types. For example, in Cassandra an integer\ncould be a \"bigint\" (a 64 bit integer), an \"int\" (a 32 bit integer), a\n\"smallint\" (a 16 bit integer), or others; in Elixir, however, integers are\njust integers (with varying size to be precise), so it is impossible to\nunivocally map Elixir integers to a specific Cassandra integer type. For this\nreason, when executing simple parameterized queries (statements) it is\nnecessary to explicitly specify the type of each value.\n\nTo specify the type of a value, that value needs to be provided as a\ntwo-element tuple where the first element is the value's type and the second\nelement is the value itself. Types are expressed with the same syntax used in\nCQL: for example, 16-bit integers are represented as `\"smallint\"`, while maps\nof strings to booleans are represented as `\"map \"`.\n\n    # Using a list of parameters:\n    statement = \"INSERT INTO species (name, properties) VALUES (?, ?)\"\n    Xandra.execute(conn, statement, [\n      {\"text\", \"human\"},\n      {\"map \", %{\"legs\" => true, \"arms\" => true, \"tail\" => false}},\n    ])\n\n    # Using a map of parameters:\n    statement = \"INSERT INTO species (name, properties) VALUES (:name, :properties)\"\n    Xandra.execute(conn, statement, %{\n      \"name\" => {\"text\", \"human\"},\n      \"properties\" => {\"map \", %{\"legs\" => true, \"arms\" => true, \"tail\" => false}},\n    })\n\nYou only need to specify types for simple queries (statements): when using\nprepared queries, the type information of each parameter of the query is\nencoded in the prepared query itself.\n\n    # Using a map of parameters:\n    prepared = Xandra.prepare!(conn, \"INSERT INTO species (name, properties) VALUES (:name, :properties)\")\n    Xandra.execute(conn, prepared, %{\n      \"name\" => \"human\",\n      \"properties\" => %{\"legs\" => true, \"arms\" => true, \"tail\" => false},\n    })\n\n##","ref":"Xandra.html#module-types"},{"type":"module","title":"User-defined types - Xandra","doc":"Xandra supports user-defined types (UDTs). A UDT can be inserted as a map with\nstring fields. For example, consider having created the following UDTs:\n\n    CREATE TYPE full_name (first_name text, last_name text)\n    CREATE TYPE profile (username text, full_name frozen )\n\nand having the following table:\n\n    CREATE TABLE users (id int PRIMARY KEY, profile frozen )\n\nInserting rows will look something like this:\n\n    prepared_insert = Xandra.prepare!(conn, \"INSERT INTO users (id, profile) VALUES (?, ?)\")\n\n    profile = %{\n      \"username\" => \"bperry\",\n      \"full_name\" => %{\"first_name\" => \"Britta\", \"last_name\" => \"Perry\"},\n    }\n    Xandra.execute!(conn, prepared_insert, [_id = 1, profile])\n\nNote that inserting UDTs is only supported on prepared queries.\n\nWhen retrieved, UDTs are once again represented as maps with string\nkeys. Retrieving the row inserted above would look like this:\n\n    %{\"profile\" => profile} = conn |> Xandra.execute!(\"SELECT id, profile FROM users\") |> Enum.fetch!(0)\n    profile\n    #=> %{\"username\" => \"bperry\", \"full_name\" => %{\"first_name\" => \"Britta\", \"last_name\" => \"Perry\"}}\n\n#","ref":"Xandra.html#module-user-defined-types"},{"type":"module","title":"Values - Xandra","doc":"Xandra supports two special values: `nil` and `:not_set`. Using `nil` explicitly\ninserts a `null` value into the Cassandra table. This is useful to **delete a value** while\ninserting. Note however that explicitly inserting `null` values into Cassandra creates so\ncalled *tombstones* which negatively affects performance and resource utilisation and is\nthus usually not recommended.\n\nThe `:not_set` value is a special value that allows to leave the value of a parametrized query\n*unset*, telling Cassandra not to insert anything for the given field. In contrast to explicit\n`null` values, no tombstone is created for this field. This is useful for prepared queries with\noptional fields. The `:not_set` value requires Cassandra native protocol v4, available since\nCassandra `2.2.x`. You can force the protocol version to v4 with the `:protocol_version`\noption.","ref":"Xandra.html#module-values"},{"type":"module","title":"Reconnections - Xandra","doc":"A Xandra connection is able to handle connection\nlosses and to automatically reconnect to Cassandra. By default, reconnections\nare retried at exponentially increasing randomized intervals, but backoff can\nbe configured through a subset of the options accepted by\n`start_link/2`.","ref":"Xandra.html#module-reconnections"},{"type":"module","title":"Clustering - Xandra","doc":"Xandra supports connecting to multiple nodes in a Cassandra cluster and\nexecuting queries on different nodes based on load balancing strategies. See\nthe documentation for `Xandra.Cluster` for more information.","ref":"Xandra.html#module-clustering"},{"type":"module","title":"Authentication - Xandra","doc":"Xandra supports Cassandra authentication. See the documentation for\n`Xandra.Authenticator` for more information.","ref":"Xandra.html#module-authentication"},{"type":"module","title":"Retrying failed queries - Xandra","doc":"Xandra takes a customizable and extensible approach to retrying failed queries\nthrough \"retry strategies\" that encapsulate the logic for retrying\nqueries. See `Xandra.RetryStrategy` for documentation on retry strategies.","ref":"Xandra.html#module-retrying-failed-queries"},{"type":"module","title":"Compression - Xandra","doc":"Xandra supports compression. To inform the Cassandra server that the\nconnections you start should use compression for data transmitted to and from\nthe server, you can pass the `:compressor` option to `start_link/1`; this\noption should be a module that implements the `Xandra.Compressor`\nbehaviour. After this, all compressed data that Cassandra sends to the\nconnection will be decompressed using this behaviour module.\n\nTo compress outgoing data (such as when issuing or preparing queries), the behavior\nof Xandra depends on the native protocol that it negotiated with the Cassandra server.\nFor native protocol v4 and earlier, the `:compressor` option should be specified\nexplicitly for every request where you want outgoing compressor. When it's specified, the\ngiven module will be used to compress data. If no `:compressor` option is\npassed, the outgoing data will not be compressed. For native protocol v5 or later,\nXandra compresses all outgoing data since compression happens at the connection level,\nso it's not necessary to pass the `:compressor` option when preparing or executing queries.\nIt's still good practice to pass it around so that your code stays agnostic of the\nnative protocol being used.","ref":"Xandra.html#module-compression"},{"type":"module","title":"Native protocol - Xandra","doc":"Xandra supports the following versions of Cassandra's native protocol:\nv5, v4, v3.\n\nBy default, Xandra will negotiate the protocol version with the Cassandra server.\nXandra will start by trying to use the highest protocol it supports (which is\nv5). If the server rejects that, then\nXandra will reconnect with the protocol advertised by the server. If you\nwant to force a specific version of the native protocol that Xandra should use,\nuse the `:protocol_version` option.","ref":"Xandra.html#module-native-protocol"},{"type":"module","title":"Logging - Xandra","doc":"Xandra connections log a few events like disconnections or connection failures.\nLogs contain the `:xandra_address` and `:xandra_port` metadata that you can\nchoose to log if desired.","ref":"Xandra.html#module-logging"},{"type":"module","title":"Custom payloads - Xandra","doc":"The Cassandra native protocol supports exchanging **custom payloads** between\nserver and client. A custom payload is a map of string keys to binary values\n(`t:custom_payload/0`).\n\nTo *send* custom payloads to the server, you can pass the `:custom_payload`\noption to functions such as `prepare/3` and `execute/4`.\n\nIf the server sends a custom payload in a response (`t:result/0`), you'll find\nit in the `:custom_payload` field of the corresponding struct (such as\n`Xandra.Page`, `Xandra.Void`, and so on).\n\nBy default, Cassandra itself ignores custom payloads sent to the server. Other\nimplementations built on top of the Cassandra native protocol might use\ncustom payloads to provide implementation-specific functionality. One such\nexample is Azure [Cosmos DB](https://azure.microsoft.com/en-us/services/cosmos-db/).","ref":"Xandra.html#module-custom-payloads"},{"type":"function","title":"Xandra.child_spec/1","doc":"Returns a child spec to use Xandra in supervision trees.\n\nTo use Xandra without passing any options you can just do:\n\n    children = [\n      Xandra,\n      # ...\n    ]\n\nIf you want to pass options, use a two-element tuple like\nusual when using child specs:\n\n    children = [\n      {Xandra, name: :xandra_connection}\n    ]","ref":"Xandra.html#child_spec/1"},{"type":"function","title":"Xandra.execute/3","doc":"Executes the given simple query, prepared query, or batch query.\n\nReturns `{:ok, result}` if executing the query was successful, or `{:error,\nerror}` otherwise. The meaning of the `params_or_options` argument depends on\nwhat `query` is:\n\n  * if `query` is a batch query, than `params_or_options` has to be a list of\n    options that will be used to run the batch query (since batch queries\n    don't use parameters as parameters are attached to each query in the\n    batch).\n\n  * if `query` is a simple query (a string) or a prepared query, then\n    `params_or_opts` is a list or map of parameters, and this function is\n    exactly the same as calling `execute(conn, query, params_or_options, [])`.\n\nWhen `query` is a batch query, successful results will always be `Xandra.Void`\nstructs. See `execute/4` for full documentation on all supported options.\n\nWhen `{:error, error}` is returned, `error` can be either a `Xandra.Error` or\na `Xandra.ConnectionError` struct. See the module documentation for more\ninformation on errors.","ref":"Xandra.html#execute/3"},{"type":"function","title":"Options for batch queries - Xandra.execute/3","doc":"When `query` is a batch query, `params_or_options` is a list of options. The following\noptions are supported:\n\n  * `:consistency` - same as the `:consistency` option described in the\n    documentation for `execute/4`.\n\n  * `:serial_consistency` - same as the `:serial_consistency` option described\n    in the documentation for `execute/4`.\n\n  * `:timestamp` - using this option means that the provided\n    timestamp will apply to all the statements in the batch that do not\n    explicitly specify a timestamp.\n\nSee `execute/4` for full documentation on all supported options if `query` is not a batch\nquery.","ref":"Xandra.html#execute/3-options-for-batch-queries"},{"type":"function","title":"Examples - Xandra.execute/3","doc":"For examples on executing simple queries or prepared queries, see the\ndocumentation for `execute/4`. Examples below specifically refer to batch\nqueries. See the documentation for `Xandra.Batch` for more information about\nbatch queries and how to construct them.\n\n    prepared_insert = Xandra.prepare!(conn, \"INSERT (email, name) INTO users VALUES (?, ?)\")\n\n    batch =\n      Xandra.Batch.new()\n      |> Xandra.Batch.add(prepared_insert, [\"abed@community.com\", \"Abed Nadir\"])\n      |> Xandra.Batch.add(prepared_insert, [\"troy@community.com\", \"Troy Barnes\"])\n      |> Xandra.Batch.add(prepared_insert, [\"britta@community.com\", \"Britta Perry\"])\n\n    # Execute the batch:\n    Xandra.execute(conn, batch)\n    #=> {:ok, %Xandra.Void{}}\n\n    # Execute the batch with a default timestamp for all statements:\n    Xandra.execute(conn, batch, timestamp: System.system_time(:millisecond) - 1_000)\n    #=> {:ok, %Xandra.Void{}}","ref":"Xandra.html#execute/3-examples"},{"type":"function","title":"Xandra.execute/4","doc":"Executes the given simple query or prepared query with the given parameters.\n\nReturns `{:ok, result}` where `result` is the result of executing `query` if\nthe execution is successful (there are no network errors or semantic errors\nwith the query), or `{:error, error}` otherwise.\n\n`result` can be one of the following:\n\n  * a `Xandra.Void` struct - returned for queries such as `INSERT`, `UPDATE`,\n    or `DELETE`.\n\n  * a `Xandra.SchemaChange` struct - returned for queries that perform changes\n    on the schema (such as creating tables).\n\n  * a `Xandra.SetKeyspace` struct - returned for `USE` queries.\n\n  * a `Xandra.Page` struct - returned for queries that return rows (such as\n    `SELECT` queries).\n\nThe properties of each of the results listed above are described in each\nresult's module.","ref":"Xandra.html#execute/4"},{"type":"function","title":"Options - Xandra.execute/4","doc":"This function supports any arbitrary option, since Xandra passes those down\nto the `Xandra.RetryStrategy` module passed in `:retry_strategy`. However, below\nis a list of the options that are specific to Xandra and that Xandra uses when executing\nthe query. Note that we might *add* options to this list in the future, which could\npotentially change the meaning of custom options you use to implement your own retry\nstrategy, and we wouldn't consider this a breaking change. Because of this, we recommend\n*scoping* custom options in your retry strategy module (for example, by prefixing them\nwith ` _ >`. When set to `:string`, UUIDs are returned in\n  their human-readable format, such as: `\"fe2b4360-28c6-11e2-81c1-0800200c9a66\"`. The default value is `:string`.\n\n* `:timeuuid_format` - Same as the `:uuid_format` option, but for values of the *timeuuid* type. The default value is `:string`.\n\n* `:telemetry_metadata` (`t:map/0`) - Custom metadata to be added to the metadata of `[:xandra, :execute_query, :start]`\n  and `[:xandra, :execute_query, :stop]` telemetry events as `extra_metadata` field. The default value is `%{}`.\n\n* `:timeout` (`t:timeout/0`) - The timeout for this call, in milliseconds. The default value is `:infinity`.","ref":"Xandra.html#execute/4-options"},{"type":"function","title":"Parameters - Xandra.execute/4","doc":"The `params` argument specifies parameters to use when executing the query; it\ncan be either a list of positional parameters (specified via `?` in the query)\nor a map of named parameters (specified as `:named_parameter` in the\nquery). When `query` is a simple query, the value of each parameter must be a\ntwo-element tuple specifying the type used to encode the value and the value\nitself; when `query` is a prepared query, this is not necessary (and values\ncan just be values) as the type information is encoded in the prepared\nquery. See the module documentation for more information about query\nparameters, types, and encoding values.","ref":"Xandra.html#execute/4-parameters"},{"type":"function","title":"Examples - Xandra.execute/4","doc":"Executing a simple query (which is just a string):\n\n    statement = \"INSERT INTO users (first_name, last_name) VALUES (:first_name, :last_name)\"\n\n    {:ok, %Xandra.Void{}} =\n      Xandra.execute(conn, statement, %{\n        \"first_name\" => {\"text\", \"Chandler\"},\n        \"last_name\" => {\"text\", \"Bing\"},\n      })\n\nExecuting the query when `atom_keys: true` has been specified in `Xandra.start_link/1`:\n\n    Xandra.execute(conn, statement, %{\n      first_name: {\"text\", \"Chandler\"},\n      last_name: {\"text\", \"Bing\"}\n    })\n\nExecuting a prepared query:\n\n    prepared = Xandra.prepare!(conn, \"INSERT INTO users (first_name, last_name) VALUES (?, ?)\")\n\n    {:ok, %Xandra.Void{}} = Xandra.execute(conn, prepared, [\"Monica\", \"Geller\"])\n\nPerforming a `SELECT` query and using `Enum.to_list/1` to convert the\n`Xandra.Page` result to a list of rows:\n\n    statement = \"SELECT * FROM users\"\n    {:ok, %Xandra.Page{} = page} = Xandra.execute(conn, statement, _params = [])\n    Enum.to_list(page)\n    #=> [%{\"first_name\" => \"Chandler\", \"last_name\" => \"Bing\"},\n    #=>  %{\"first_name\" => \"Monica\", \"last_name\" => \"Geller\"}]\n\nPerforming the query when `atom_keys: true` has been specified in `Xandra.start_link/1`:\n\n    {:ok, page} = Xandra.execute(conn, statement, _params = [])\n    Enum.to_list(page)\n    #=> [%{first_name:  \"Chandler\", last_name: \"Bing\"},\n    #=>  %{first_name: \"Monica\", last_name: \"Geller\"}]\n\nEnsuring the write is written to the commit log and memtable of at least three replica nodes:\n\n    statement = \"INSERT INTO users (first_name, last_name) VALUES ('Chandler', 'Bing')\"\n    {:ok, %Xandra.Void{}} = Xandra.execute(conn, statement, _params = [], consistency: :three)","ref":"Xandra.html#execute/4-examples"},{"type":"function","title":"Paging - Xandra.execute/4","doc":"Since `execute/4` supports the `:paging_state` option, it is possible to manually\nimplement paging. For example, given the following prepared query:\n\n    prepared = Xandra.prepare!(conn, \"SELECT first_name FROM users\")\n\nWe can now execute such query with a specific page size using the `:page_size`\noption:\n\n    {:ok, %Xandra.Page{} = page} = Xandra.execute(conn, prepared, [], page_size: 2)\n\nSince `:page_size` is `2`, `page` will contain at most `2` rows:\n\n    Enum.to_list(page)\n    #=> [%{\"first_name\" => \"Ross\"}, %{\"first_name\" => \"Rachel\"}]\n\nNow, we can pass `page.paging_state` as the value of the `:paging_state`\noption to let the paging start from where we left off:\n\n    {:ok, %Xandra.Page{} = new_page} =\n      Xandra.execute(conn, prepared, [], page_size: 2, paging_state: page.paging_state)\n\n    Enum.to_list(page)\n    #=> [%{\"first_name\" => \"Joey\"}, %{\"first_name\" => \"Phoebe\"}]\n\nHowever, using `:paging_state` and `:page_size` directly with `execute/4` is not\nrecommended when the intent is to \"stream\" a query. For that, it's recommended\nto use `stream_pages!/4`. Also note that if the `:paging_state` option is set to `nil`,\nmeaning there are no more pages to fetch, an `ArgumentError` exception will be raised;\nbe sure to check for this with `page.paging_state != nil`.","ref":"Xandra.html#execute/4-paging"},{"type":"function","title":"Tracing - Xandra.execute/4","doc":"Cassandra supports [tracing queries](https://docs.datastax.com/en/cql/3.3/cql/cql_reference/cqlshTracing.html).\nIf you set the `:tracing` option to `true`, the executed query will be traced.\nThis means that a tracing ID (a binary UUID) will be set in the response of the query\nand that Cassandra will write relevant tracing events to tracing-related tables in the\n`system_traces` keyspace.\n\nIn Xandra, all response structs contain an accessible `tracing_id` field that is set\nto `nil` except for when tracing is enabled. In those cases, `tracing_id` is a binary\nUUID that you can use to select events from the traces tables.\n\nFor example:\n\n    {:ok, page} = Xandra.execute(conn, \"SELECT * FROM users\", [], tracing: true)\n\n    statement = \"SELECT * FROM system_traces.events WHERE session_id = ?\"\n    {:ok, trace_events_page} = Xandra.execute(conn, statement, [{\"uuid\", page.tracing_id}])\n\nNote that tracing is an expensive operation for Cassandra that puts load on\nexecuting queries. This is why this option is only supported *per-query* in\n`execute/4` instead of connection-wide.","ref":"Xandra.html#execute/4-tracing"},{"type":"function","title":"Xandra.execute!/3","doc":"Executes the given simple query, prepared query, or batch query, raising if\nthere's an error.\n\nThis function behaves exactly like `execute/3`, except that it returns\nsuccessful results directly and raises on errors.","ref":"Xandra.html#execute!/3"},{"type":"function","title":"Examples - Xandra.execute!/3","doc":"Xandra.execute!(conn, \"INSERT INTO users (name, age) VALUES ('Jane', 29)\")\n    #=> %Xandra.Void{}","ref":"Xandra.html#execute!/3-examples"},{"type":"function","title":"Xandra.execute!/4","doc":"Executes the given simple query, prepared query, or batch query, raising if\nthere's an error.\n\nThis function behaves exactly like `execute/4`, except that it returns\nsuccessful results directly and raises on errors.","ref":"Xandra.html#execute!/4"},{"type":"function","title":"Examples - Xandra.execute!/4","doc":"statement = \"INSERT INTO users (name, age) VALUES ('John', 43)\"\n    Xandra.execute!(conn, statement, _params = [], consistency: :quorum)\n    #=> %Xandra.Void{}","ref":"Xandra.html#execute!/4-examples"},{"type":"function","title":"Xandra.prepare/3","doc":"Prepares the given query.\n\nThis function prepares the given statement on the Cassandra server. If\npreparation is successful and there are no network errors while talking to the\nserver, `{:ok, prepared}` is returned, otherwise `{:error, error}` is\nreturned.\n\nThe returned prepared query can be run through `execute/4`, or used inside a\nbatch (see `Xandra.Batch`).\n\nErrors returned by this function can be either `Xandra.Error` or\n`Xandra.ConnectionError` structs. See the module documentation for more\ninformation about errors.","ref":"Xandra.html#prepare/3"},{"type":"function","title":"Options - Xandra.prepare/3","doc":"* `:compressor` - The compressor module to use for compressing and decompressing data.\n  See the \"Compression\" section in the module documentation. By default\n  this option is not present, which means no compression is used.\n\n* `:force` (`t:boolean/0`) - When `true`, forces the preparation of the query on\n  the server instead of trying to read the prepared query from cache. See\n  the \"Prepared queries cache\" section below. The default value is `false`.\n\n* `:tracing` (`t:boolean/0`) - Turn on tracing for the preparation of the\n  given query and sets the `tracing_id` field in the returned prepared\n  query. See the \"Tracing\" option in `execute/4`. The default value is `false`.\n\n* `:custom_payload` - A custom payload to send to the Cassandra server alongside the request. Only\n  supported in `QUERY`, `PREPARE`, `EXECUTE`, and `BATCH` requests. The custom\n  payload must be of type `t:custom_payload/0`. See the\n  [\"Custom payloads\"](#module-custom-payloads) section in the module documentation.\n\n* `:telemetry_metadata` (`t:map/0`) - Custom metadata to be added to the metadata of `[:xandra, :prepare_query, :start]`\n  and `[:xandra, :prepare_query, :stop]` telemetry events as `extra_metadata` field. The default value is `%{}`.\n\n* `:timeout` (`t:timeout/0`) - The timeout for this call, in milliseconds. The default value is `:infinity`.","ref":"Xandra.html#prepare/3-options"},{"type":"function","title":"Prepared queries cache - Xandra.prepare/3","doc":"Since Cassandra prepares queries on a per-node basis (and not on a\nper-connection basis), Xandra internally caches prepared queries for each\nconnection or pool of connections. This means that if you prepare a query that\nwas already prepared, no action will be executed on the Cassandra server and\nthe prepared query will be returned from the cache.\n\nIf the Cassandra node goes down, however, the prepared query will be\ninvalidated and trying to use the one from cache will result in a\n`Xandra.Error`. However, this is automatically handled by Xandra: when such an\nerror is returned, Xandra will first retry to prepare the query and only\nreturn an error if the preparation fails.\n\nIf you want to ensure a query is prepared on the server, you can set the\n`:force` option to `true`.","ref":"Xandra.html#prepare/3-prepared-queries-cache"},{"type":"function","title":"Examples - Xandra.prepare/3","doc":"{:ok, prepared} = Xandra.prepare(conn, \"SELECT * FROM users WHERE id = ?\")\n    {:ok, _page} = Xandra.execute(conn, prepared, [_id = 1])\n\n    {:error, %Xandra.Error{reason: :invalid_syntax}} = Xandra.prepare(conn, \"bad syntax\")\n\n    # Force a query to be prepared on the server and not be read from cache:\n    Xandra.prepare!(conn, \"SELECT * FROM users WHERE ID = ?\", force: true)","ref":"Xandra.html#prepare/3-examples"},{"type":"function","title":"Xandra.prepare!/3","doc":"Prepares the given query, raising if there's an error.\n\nThis function works exactly like `prepare/3`, except it returns the prepared\nquery directly if preparation succeeds, otherwise raises the returned error.","ref":"Xandra.html#prepare!/3"},{"type":"function","title":"Examples - Xandra.prepare!/3","doc":"prepared = Xandra.prepare!(conn, \"SELECT * FROM users WHERE id = ?\")\n    {:ok, _page} = Xandra.execute(conn, prepared, [_id = 1])","ref":"Xandra.html#prepare!/3-examples"},{"type":"function","title":"Xandra.start_link/1","doc":"Starts a new pool of connections to Cassandra.\n\nThis function starts a new connection to the provided Cassandra node.","ref":"Xandra.html#start_link/1"},{"type":"function","title":"Options - Xandra.start_link/1","doc":"These are the options supported by this function:\n\n* `:atom_keys` (`t:boolean/0`) - Whether or not results of and parameters to `execute/4` will have atom\n  keys. If `true`, the result maps will have column names returned as\n  atoms rather than as strings. Additionally, maps that represent named\n  parameters will need atom keys. The default value is `false`.\n\n* `:authentication` (tuple with `t:module/0` and `t:term/0`) - Two-element tuple: the authenticator module to use for authentication\n  and its supported options. See `Xandra.Authenticator`.\n\n* `:backoff_min` (`t:non_neg_integer/0`) - The minimum backoff interval (in milliseconds). The default value is `1000`.\n\n* `:backoff_max` (`t:non_neg_integer/0`) - The maximum backoff interval (in milliseconds). The default value is `30000`.\n\n* `:backoff_type` - The backoff strategy. `:stop` means the connection will stop when a disconnection happens,\n  `:exp` means exponential backoff, `:rand` is random backoff, and `:rand_exp` is random\n  exponential backoff. The default value is `:rand_exp`.\n\n* `:compressor` (`t:module/0`) - The compressor module to use for compressing and decompressing data.\n  See the [\"Compression\" section](#module-compression) in the module documentation. By default\n  this option is not present, which means no compression is used.\n\n* `:configure` (`t:configure_fun/0`) - A function to run before every connect attempt to dynamically configure the options. It's\n  either a 1-arity fun which gets called with the options and must return options, or a\n  `{module, function, args}` tuple with the options prepended to `args` (the return value\n  of the function must still be options). By default, options are not modified. This\n  function is called in the connection process. *Available since v0.18.0*.\n\n* `:connect_timeout` (`t:timeout/0`) - Time to wait (in milliseconds) for Xandra to establish a network connection.\n  If the timeout is reached, the connection considers it as a failed connection attempt\n  and behaves according to the `:backoff_type` option. *Available since v0.18.0*. The default value is `5000`.\n\n* `:default_consistency` (`t:consistency/0`) - The default consistency to set for all queries. Can be overridden on a per-query\n  basis through the `:consistency` option in `execute/4`. The default value is `:one`.\n\n* `:debug` (`t:term/0`)\n\n* `:encryption` (`t:boolean/0`) - Whether to connect to Cassandra using SSL. If you want to set up SSL\n  options, see the `:transport_options` option. The default value is `false`.\n\n* `:keyspace` (`t:String.t/0`) - `USE` this keyspace right after establishing a connection to the server. This effectively\n  replaces most uses of the `:after_connect` option that was available before v0.18.0.\n  *Available since v0.18.0*.\n\n* `:name` (`t:term/0`) - Name registration, just like `GenServer`.\n\n* `:nodes` (list of `t:String.t/0`) - The Cassandra node to connect to. This option is a list for consistency with\n  `Xandra.Cluster`, but if using `Xandra` directly, it can only contain a single node.\n  Such node can have the form `\"ADDRESS:PORT\"`, or `\"ADDRESS\"` (port defaults to\n  `9042`). See the documentation for `Xandra.Cluster` for more information on\n  connecting to multiple nodes. The default value is `[\"127.0.0.1\"]`.\n\n* `:protocol_version` (`t:native_protocol_version/0`) - The enforced version of the Cassandra native protocol to use. If this option\n  is not present, Xandra will negotiate the protocol with the server, starting\n  with the most recent one and falling back to older ones if needed. Must be one\n  of `:v5`, `:v4`, `:v3`. See\n  the [relevant section](#module-native-protocol) in the module documentation.\n\n* `:show_sensitive_data_on_connection_error` (`t:boolean/0`) - Is it ok to show sensitive data on connection errors? Useful for\n  debugging and in tests. The default value is `false`.\n\n* `:transport_options` (`t:keyword/0`) - Options to forward to the socket transport. If the `:encryption` option is `true`,\n  then the transport is SSL (see the Erlang `:ssl` module) otherwise it's\n  TCP (see the `:gen_tcp` Erlang module).","ref":"Xandra.html#start_link/1-options"},{"type":"function","title":"Examples - Xandra.start_link/1","doc":"# Start a connection:\n    {:ok, conn} = Xandra.start_link()\n\n    # Start a connection and register it under a name:\n    {:ok, _conn} = Xandra.start_link(name: :xandra)\n\nIf you're using Xandra under a supervisor, see `child_spec/1`.","ref":"Xandra.html#start_link/1-examples"},{"type":"function","title":"Xandra.start_link_opts_schema/0","doc":"Returns the `NimbleOptions` schema used for validating the options for `start_link/1`.\n\nThis function is meant to be used by other libraries that want to extend Xandra's\nfunctionality, and that rely on `start_link/1` at some point. The keys in the returned\nschema are not all public, so you should always refer to the documentation for\n`start_link/1`.","ref":"Xandra.html#start_link_opts_schema/0"},{"type":"function","title":"Examples - Xandra.start_link_opts_schema/0","doc":"iex> schema = Xandra.start_link_opts_schema()\n    iex> schema[:encryption][:type]\n    :boolean","ref":"Xandra.html#start_link_opts_schema/0-examples"},{"type":"function","title":"Xandra.stop/3","doc":"Synchronously stops the given connection with the given reason.\n\nWaits `timeout` milliseconds for the connection to stop before aborting and exiting.","ref":"Xandra.html#stop/3"},{"type":"function","title":"Xandra.stream_pages!/4","doc":"Streams the results of a simple query or a prepared query with the given `params`.\n\nThis function can be used to stream the results of `query` so as not to load\nthem entirely in memory. This function doesn't send any query to Cassandra\nright away: it will only execute queries as necessary when results are\nrequested out of the returned stream.\n\nThe returned value is a stream of `Xandra.Page` structs, where each of such\nstructs contains at most as many rows as specified by the `:page_size`\noption. Every time an element is requested from the stream, `query` will be\nexecuted with `params` to get that result.\n\nIn order to get each result from Cassandra, `execute!/4` is used: this means\nthat if there is an error (such as a network error) when executing the\nqueries, that error will be raised.\n\n#","ref":"Xandra.html#stream_pages!/4"},{"type":"function","title":"Simple or prepared queries - Xandra.stream_pages!/4","doc":"Regardless of `query` being a simple query or a prepared query, this function\nwill execute it every time a result is needed from the returned stream. For\nthis reason, it is usually a good idea to use prepared queries when streaming.","ref":"Xandra.html#stream_pages!/4-simple-or-prepared-queries"},{"type":"function","title":"Options - Xandra.stream_pages!/4","doc":"`options` supports all the options supported by `execute/4`, with the same\ndefault values.","ref":"Xandra.html#stream_pages!/4-options"},{"type":"function","title":"Examples - Xandra.stream_pages!/4","doc":"prepared = Xandra.prepare!(conn, \"SELECT * FROM users\")\n    users_stream = Xandra.stream_pages!(conn, prepared, _params = [], page_size: 2)\n\n    [%Xandra.Page{} = _page1, %Xandra.Page{} = _page2] = Enum.take(users_stream, 2)","ref":"Xandra.html#stream_pages!/4-examples"},{"type":"type","title":"Xandra.configure_fun/0","doc":"Possible values for the `:configure` option in `start_link/1`.","ref":"Xandra.html#t:configure_fun/0"},{"type":"type","title":"Xandra.conn/0","doc":"A single Xandra connection process.","ref":"Xandra.html#t:conn/0"},{"type":"type","title":"Xandra.consistency/0","doc":"Consistency level for queries.\n\nThis is supported as the default consistency on the connection level (see `start_link/1`)\nor as a per-query consistency (see `execute/3` and `execute/4`).","ref":"Xandra.html#t:consistency/0"},{"type":"type","title":"Xandra.custom_payload/0","doc":"Custom payload that Xandra can exchange with the server.\n\nSee the [\"Custom payloads\"](#module-custom-payloads) section in the\nmodule documentation.","ref":"Xandra.html#t:custom_payload/0"},{"type":"type","title":"Xandra.error/0","doc":"An error that can be returned from a query.\n\nThis is either a semantic error returned by Cassandra or a connection error.","ref":"Xandra.html#t:error/0"},{"type":"type","title":"Xandra.native_protocol_version/0","doc":"Cassandra native protocol version.","ref":"Xandra.html#t:native_protocol_version/0"},{"type":"type","title":"Xandra.result/0","doc":"The result of a query.","ref":"Xandra.html#t:result/0"},{"type":"type","title":"Xandra.start_option/0","doc":"Options for `start_link/1`.","ref":"Xandra.html#t:start_option/0"},{"type":"type","title":"Xandra.statement/0","doc":"A statement (query) to pass to `execute/4` and other functions.","ref":"Xandra.html#t:statement/0"},{"type":"type","title":"Xandra.values/0","doc":"","ref":"Xandra.html#t:values/0"},{"type":"behaviour","title":"Xandra.Authenticator","doc":"A behaviour module for implementing a Cassandra authenticator.","ref":"Xandra.Authenticator.html"},{"type":"behaviour","title":"Examples - Xandra.Authenticator","doc":"defmodule MyAuthenticator do\n      @behaviour Xandra.Authenticator\n\n      def response_body(options) do\n        [\"user:\", Keyword.fetch!(options, :user), \"_password:\", Keyword.fetch!(options, :password)]\n      end\n    end\n\nTo use the authenticator defined above:\n\n    Xandra.start_link(authentication: {MyAuthenticator, user: \"foo\", password: \"bar\"})\n\nXandra supports Cassandra's PasswordAuthenticator by default, see\n`Xandra.Authenticator.Password` for more information.","ref":"Xandra.Authenticator.html#module-examples"},{"type":"callback","title":"Xandra.Authenticator.response_body/1","doc":"Returns an iodata that's used as the response body to Cassandra's auth challenge.","ref":"Xandra.Authenticator.html#c:response_body/1"},{"type":"module","title":"Xandra.Authenticator.Password","doc":"A `Xandra.Authenticator` that implements support for PasswordAuthenticator\nto authenticate with Cassandra server.","ref":"Xandra.Authenticator.Password.html"},{"type":"module","title":"Example - Xandra.Authenticator.Password","doc":"options = [username: \"xandra\", password: \"secret\"]\n\n    Xandra.start_link(authentication: {Xandra.Authenticator.Password, options})","ref":"Xandra.Authenticator.Password.html#module-example"},{"type":"module","title":"Xandra.Batch","doc":"Represents a batch of simple and/or prepared queries.\n\nThis module provides a data structure that can be used to group queries and\nexecute them as a Cassandra `BATCH` query. Batch queries can be executed\nthrough `Xandra.execute/3` and `Xandra.execute!/3`; see their respective\ndocumentation for more information.\n\nNote that the `t/1` type is not documented as it's not meant\nfor public use. If you want to use batches in your typespecs, use `t:t/0`.","ref":"Xandra.Batch.html"},{"type":"function","title":"Xandra.Batch.add/3","doc":"Adds a query to the given `batch`.\n\n`query` has to be either a simple query (statement) or a prepared query. Note\nthat parameters have to be added alongside their corresponding query when\nadding a query to a batch. In contrast with functions like `Xandra.execute/4`,\nsimple queries in batch queries only support positional parameters and **do\nnot** support named parameters; this is a current Cassandra limitation. If a\nmap of named parameters is passed alongside a simple query, an `ArgumentError`\nexception is raised. Named parameters are supported with prepared queries.","ref":"Xandra.Batch.html#add/3"},{"type":"function","title":"Examples - Xandra.Batch.add/3","doc":"prepared = Xandra.prepare!(conn, \"INSERT INTO users (name, age) VALUES (?, ?)\")\n\n    batch =\n      Xandra.Batch.new()\n      |> Xandra.Batch.add(prepared, [\"Rick\", 60])\n      |> Xandra.Batch.add(prepared, [\"Morty\", 14])\n      |> Xandra.Batch.add(prepared, [\"Jerry\", 35])\n      |> Xandra.Batch.add(\"DELETE FROM users WHERE name = 'Jerry'\")\n\n    Xandra.execute!(conn, batch)","ref":"Xandra.Batch.html#add/3-examples"},{"type":"function","title":"Xandra.Batch.new/1","doc":"Creates a new batch query.\n\n`type` represents the type of the batch query (`:logged`, `:unlogged`, or\n`:counter`). See the Cassandra documentation for the meaning of these types.","ref":"Xandra.Batch.html#new/1"},{"type":"function","title":"Examples - Xandra.Batch.new/1","doc":"batch = Xandra.Batch.new()","ref":"Xandra.Batch.html#new/1-examples"},{"type":"type","title":"Xandra.Batch.t/0","doc":"","ref":"Xandra.Batch.html#t:t/0"},{"type":"type","title":"Xandra.Batch.type/0","doc":"","ref":"Xandra.Batch.html#t:type/0"},{"type":"module","title":"Xandra.Cluster","doc":"Connection to a Cassandra cluster.\n\nThis module is a \"proxy\" connection with support for connecting to multiple\nnodes in a Cassandra cluster and executing queries on such nodes based on a\ngiven *policy*.","ref":"Xandra.Cluster.html"},{"type":"module","title":"Usage - Xandra.Cluster","doc":"This module manages pools of connections to different nodes in a Cassandra cluster.\nEach pool is a pool of `Xandra` connections to a specific node.\n\nThe API provided by this module mirrors the API provided by the `Xandra`\nmodule. Queries executed through this module will be \"routed\" to nodes\nin the provided list of nodes based on a policy. See the\n[\"Load balancing policies\" section](#module-load-balancing-policies).\n\nRegardless of the underlying pool, `Xandra.Cluster` will establish\none extra connection to a node in the cluster for internal purposes.\nWe refer to this connection as the **control connection**.\n\nHere is an example of how one could use `Xandra.Cluster` to connect to a cluster:\n\n    Xandra.Cluster.start_link(\n      nodes: [\"cassandra1.example.net\", \"cassandra2.example.net\"],\n      pool_size: 10,\n    )\n\nThe code above will establish a pool of ten connections to each of the nodes\nspecified in `:nodes`, plus one extra connection used for internal\npurposes, for a total of twenty-one connections going out of the machine.","ref":"Xandra.Cluster.html#module-usage"},{"type":"module","title":"Child Specification - Xandra.Cluster","doc":"`Xandra.Cluster` implements a `child_spec/1` function, so it can be used as a child\nunder a supervisor:\n\n    children = [\n      # ...,\n      {Xandra.Cluster, nodes: [\"cassandra-seed.example.net\"]}\n    ]","ref":"Xandra.Cluster.html#module-child-specification"},{"type":"module","title":"Contact Points and Cluster Discovery - Xandra.Cluster","doc":"`Xandra.Cluster` auto-discovers peer nodes in the cluster, by using the `system.peers`\nbuilt-in Cassandra table. Once Xandra discovers peers, it opens a pool of connections\nto a subset of the peers based on the chosen load-balancing policy (see below).\n\nThe `:nodes` option in `start_link/1` specifies the **contact points**. The contact\npoints are used to discover the rest of the nodes in the cluster. It's generally\na good idea to provide multiple contacts points, so that if some of those are\nunreachable, the others can be used to discover the rest of the cluster. `Xandra.Cluster`\ntries to connect to contact points in the order they are specified in the `:nodes`\noption, initially ignoring the chosen load-balancing policy. Once a connection is\nestablished, then that contact point is used to discover the rest of the cluster\nand open connection pools according to the load-balancing policy.\n\nXandra also **refreshes** the cluster topology periodically. See the\n`:refresh_topology_interval` option in `start_link/1`.","ref":"Xandra.Cluster.html#module-contact-points-and-cluster-discovery"},{"type":"module","title":"Load-balancing Policies - Xandra.Cluster","doc":"`Xandra.Cluster` uses customizable \"load-balancing policies\" to manage nodes\nin the cluster. A load-balancing policy is a module that implements the\n`Xandra.Cluster.LoadBalancing` behaviour. Xandra uses load-balancing policies\nfor these purposes:\n\n  * Choosing which node to execute a query on\n  * Choosing which nodes to open pools of connections to (see the `:target_pools`\n    option in `start_link/1`)\n  * Choosing which node the control connection connects to (or **re-connects** to\n    in case of disconnections)\n\nXandra ships with the following built-in load-balancing policies:\n\n  * `Xandra.Cluster.LoadBalancingPolicy.Random` - it will choose one of the\n    connected nodes at random and execute the query on that node.\n  * `Xandra.Cluster.LoadBalancingPolicy.DCAwareRoundRobin` - it will execute the\n    queries on the nodes in a round robin manner, prioritizing the current DC.","ref":"Xandra.Cluster.html#module-load-balancing-policies"},{"type":"module","title":"Disconnections and Reconnections - Xandra.Cluster","doc":"`Xandra.Cluster` also supports nodes disconnecting and reconnecting: if Xandra\ndetects one of the nodes in `:nodes` going down, it will not execute queries\nagainst it anymore, but will start executing queries on it as soon as it\ndetects such node is back up.\n\nIf all specified nodes happen to be down when a query is executed, a\n`Xandra.ConnectionError` with reason `{:cluster, :not_connected}` will be\nreturned.","ref":"Xandra.Cluster.html#module-disconnections-and-reconnections"},{"type":"module","title":"Telemetry - Xandra.Cluster","doc":"`Xandra.Cluster` emits several Telemetry events to help you log, instrument,\nand debug your application. See the [*Telemetry Events*](telemetry-events.html)\npage in the guides for a comprehensive list of the events that Xandra emits.","ref":"Xandra.Cluster.html#module-telemetry"},{"type":"function","title":"Xandra.Cluster.connected_hosts/1","doc":"Returns a list of hosts that the cluster has outgoing connections to.","ref":"Xandra.Cluster.html#connected_hosts/1"},{"type":"function","title":"Xandra.Cluster.execute/3","doc":"Same as `execute/4` but with optional arguments.","ref":"Xandra.Cluster.html#execute/3"},{"type":"function","title":"Xandra.Cluster.execute/4","doc":"Executes a query on a node in the cluster.\n\nThis function executes a query on a node in the cluster. The node is chosen based\non the load-balancing policy given in `start_link/1`.\n\nSupports the same options as `Xandra.execute/4`. In particular, the `:retry_strategy`\noption is cluster-aware, meaning that queries are retried on possibly different nodes\nin the cluster.","ref":"Xandra.Cluster.html#execute/4"},{"type":"function","title":"Xandra.Cluster.execute!/3","doc":"Same as `execute/3` but returns the result directly or raises in case of errors.","ref":"Xandra.Cluster.html#execute!/3"},{"type":"function","title":"Xandra.Cluster.execute!/4","doc":"Same as `execute/4` but returns the result directly or raises in case of errors.","ref":"Xandra.Cluster.html#execute!/4"},{"type":"function","title":"Xandra.Cluster.prepare/3","doc":"Same as `Xandra.prepare/3`.\n\nPreparing a query through `Xandra.Cluster` will prepare it only on one node,\naccording to the load-balancing policy chosen in `start_link/1`. To prepare\nand execute a query on the same node, you could use `run/3`:\n\n    Xandra.Cluster.run(cluster, fn conn ->\n      # \"conn\" is the pool of connections for a specific node.\n      prepared = Xandra.prepare!(conn, \"SELECT * FROM system.local\")\n      Xandra.execute!(conn, prepared, _params = [])\n    end)\n\nThanks to the prepared query cache, we can always reprepare the query and execute\nit because after the first time (on each node) the prepared query will be fetched\nfrom the cache. However, if a prepared query is unknown on a node, Xandra will\nprepare it on that node on the fly, so we can simply do this as well:\n\n    prepared = Xandra.Cluster.prepare!(cluster, \"SELECT * FROM system.local\")\n    Xandra.Cluster.execute!(cluster, prepared, _params = [])\n\nNote that this goes through the cluster twice, so there's a high chance that\nthe query will be prepared on one node and then executed on another node.\nThis is however useful if you want to use the `:retry_strategy` option in\n`execute!/4`: in the `run/3` example above, if you use `:retry_strategy` with\n`Xandra.execute!/3`, the query will be retried on the same pool of connections\nto the same node. `execute!/4` will retry queries going through the cluster\nagain instead.","ref":"Xandra.Cluster.html#prepare/3"},{"type":"function","title":"Xandra.Cluster.prepare!/3","doc":"Same as `prepare/3` but raises in case of errors.\n\nIf the function is successful, the prepared query is returned directly\ninstead of in an `{:ok, prepared}` tuple like in `prepare/3`.","ref":"Xandra.Cluster.html#prepare!/3"},{"type":"function","title":"Xandra.Cluster.run/3","doc":"Runs a function with a given connection.\n\nThe connection that is passed to `fun` is a Xandra connection, not a\ncluster. This means that you should call `Xandra` functions on it.\nSince the connection is a single connection, it means that it's a connection\nto a specific node, so you can do things like prepare a query and then execute\nit because you can be sure it's prepared on the same node where you're\nexecuting it.","ref":"Xandra.Cluster.html#run/3"},{"type":"function","title":"Examples - Xandra.Cluster.run/3","doc":"query = \"SELECT * FROM system_schema.keyspaces\"\n\n    Xandra.Cluster.run(cluster, fn conn ->\n      prepared = Xandra.prepare!(conn, query)\n      Xandra.execute!(conn, prepared, _params = [])\n    end)","ref":"Xandra.Cluster.html#run/3-examples"},{"type":"function","title":"Xandra.Cluster.start_link/1","doc":"Starts connections to a cluster.","ref":"Xandra.Cluster.html#start_link/1"},{"type":"function","title":"Options - Xandra.Cluster.start_link/1","doc":"This function accepts all options accepted by `Xandra.start_link/1` and\nand forwards them to each underlying connection or pool of connections. The following\noptions are specific to this function:\n\n* `:nodes` (list of `t:String.t/0`) - A list of nodes to use as *contact points* when setting up the cluster. Each node in this\n  list must be a hostname (`\"cassandra.example.net\"`), IPv4 (`\"192.168.0.100\"`),\n  or IPv6 (`\"16:64:c8:0:2c:58:5c:c7\"`) address. An optional port can be specified by\n  including `: ` after the address, such as `\"cassandra.example.net:9876\"`.\n  See the [*Contact points and cluster discovery*\n  section](#module-contact-points-and-cluster-discovery) in the module documentation. The default value is `[\"127.0.0.1\"]`.\n\n* `:load_balancing` (`{module(), term()}` or `:random`) - Load balancing \"policy\". See the [*Load balancing policies*\n  section](#module-load-balancing-policies) in the module documentation.\n  The policy must be expressed as a `{module, options}` tuple, where `module`\n  is a module that implements the `Xandra.Cluster.LoadBalancingPolicy` behaviour, and\n  `options` is any term that is passed to the `c:Xandra.Cluster.LoadBalancingPolicy.init/1`\n  callback. This option changed in v0.15.0.\n  Before *v0.15.0*, the only supported values were `:priority` and `:random`.\n  `:random` is deprecated in favor of using `{Xandra.Cluster.LoadBalancingPolicy.Random, []}`.\n  `:priority` has been removed. The default value is `:random`.\n\n* `:autodiscovery` (`t:boolean/0`) - (**deprecated**) Whether to enable autodiscovery. Since v0.15.0, this option is deprecated\n  and autodiscovery is always enabled.\n\n* `:autodiscovered_nodes_port` (`t::inet.port_number/0`) - The port to use when connecting to autodiscovered nodes. Cassandra does not advertise\n  the port of nodes when discovering them, so you'll need to specify one explicitly.\n  This might get fixed in future Cassandra versions. The default value is `9042`.\n\n* `:refresh_topology_interval` (`t:timeout/0`) - The interval at which Xandra will refresh the cluster topology by querying the control\n  connection to discover peers. When the connection refreshes the topology, it will\n  also start and stop pools for new and removed nodes, effectively \"syncing\" with\n  the cluster. *Available since v0.15.0*. The default value is `300000`.\n\n* `:target_pools` (`t:pos_integer/0`) - The number of nodes to start pools to. Each pool will use the `:pool_size` option to\n  determine how many single connections to open to that node. This number is a *target*\n  number, which means that sometimes there might not be enough nodes to start this many\n  pools. Xandra won't ever start more than `:target_pools` pools. *Available since v0.15.0*. The default value is `2`.\n\n* `:name` (`t:term/0`) - The name to register this cluster under. Follows the name registration rules of `GenServer`.\n\n* `:sync_connect` - Whether to wait for at least one connection to a node in the cluster to be established\n  before returning from `start_link/1`. If `false`, connecting is async, which means that\n  even if `start_link/1` returns `{:ok, pid}`, that's the PID of the cluster process,\n  which has not necessarily established any connections yet. If this option is\n  an integer or `:infinity` (that is, a term of type `t:timeout/0`), then this function\n  only returns when at least one node connection is established. If the timeout expires,\n  this function returns `{:error, :sync_connect_timeout}`. *Available since v0.16.0*.\n\n  This is only useful in rare cases when you want to make sure that the has connected\n  at least once before returning from `start_link/1`. This is fragile though, because\n  the cluster could connect once and then drop connections right away, so this doesn't\n  mean that the cluster is connected, but rather that it *connected at least once*.\n  This is useful, for example, in test suites where you're not worried about\n  resiliency but rather race conditions. In most cases, the\n  `:queue_checkouts_before_connecting` option is what you want.\n\n  The default value is `false`.\n\n* `:queue_checkouts_before_connecting` (`t:keyword/0`) - Controls how to handle checkouts that go through the cluster *before* the cluster\n  is able to establish a connection to **any node**. Whenever you run a cluster function,\n  the cluster checks out a connection from one of the connected nodes and executes the\n  request on that connection. However, if you try to run any cluster function before the\n  cluster connects to any of the nodes, you'll likely get `Xandra.ConnectionError`s\n  with reason `{:cluster, :not_connected}`. This is because the cluster needs to establish\n  at least one connection to one node before it can execute requests. This option addresses\n  this issue by queueing \"checkout requests\" until the cluster establishes a connection\n  to a node. Once the connection is established, the cluster starts to hand over\n  connections. If you want to **disable this behavior**, set `:max_size` to `0`. *Available\n  since v0.18.0*. This option supports the following sub-options: The default value is `[]`.\n\n  * `:max_size` (`t:non_neg_integer/0`) - The number of checkouts to queue in the cluster and flush as soon as a connection\n    is established. The default value is `100`.\n\n  * `:timeout` (`t:timeout/0`) - How long to hold on to checkout requests for. When this timeout expires, all requests\n    are dropped and a connection error is returned to each caller. The default value is `5000`.\n\n* `:pool_size` (`t:pos_integer/0`) - The number of connections to open to each node in the cluster. *Available since v0.18.0*. The default value is `1`.\n\n* `:debug` (`t:term/0`) - Same as the `:debug` option in `GenServer.start_link/3`. *Available since v0.18.0*.\n\n* `:spawn_opt` (`t:term/0`) - Same as the `:spawn_opt` option in `GenServer.start_link/3`. *Available since v0.18.0*.\n\n* `:hibernate_after` (`t:term/0`) - Same as the `:hibernate_after` option in `GenServer.start_link/3`.\n  *Available since v0.18.0*.\n\n\n\n> ##","ref":"Xandra.Cluster.html#start_link/1-options"},{"type":"function","title":"Control connection {: .neutral} - Xandra.Cluster.start_link/1","doc":">\n> A `Xandra.Cluster` starts **one additional \"control connection\"** to one of the\n> nodes in the cluster. This could be a node in the given `:nodes` (a *contact point*)\n> or a discovered peer in the cluster. See the [*Contact points and cluster discovery*\n> section](#module-contact-points-and-cluster-discovery) in the module documentation.","ref":"Xandra.Cluster.html#start_link/1-control-connection-neutral"},{"type":"function","title":"Examples - Xandra.Cluster.start_link/1","doc":"Starting a Xandra cluster using two nodes as the contact points:\n\n    {:ok, cluster} =\n      Xandra.Cluster.start_link(nodes: [\"cassandra1.example.net\", \"cassandra2.example.net\"])\n\nStarting a pool of five connections to each node in the same cluster as the given\ncontact point:\n\n    {:ok, cluster} =\n      Xandra.Cluster.start_link(nodes: [\"cassandra-seed.example.net\"], pool_size: 5)\n\nPassing options down to each connection:\n\n    {:ok, cluster} =\n      Xandra.Cluster.start_link(\n        nodes: [\"cassandra.example.net\"],\n        keyspace: \"my_keyspace\"\n      )","ref":"Xandra.Cluster.html#start_link/1-examples"},{"type":"function","title":"Xandra.Cluster.stop/3","doc":"Synchronously stops the given cluster with the given reason.\n\nWaits `timeout` milliseconds for the cluster to stop before aborting and exiting.","ref":"Xandra.Cluster.html#stop/3"},{"type":"function","title":"Xandra.Cluster.stream_pages!/4","doc":"Returns a stream of pages.\n\nWhen streaming pages through a cluster, the streaming is done\nfrom a single node, that is, this function just calls out to\n`Xandra.stream_pages!/4` after choosing a node appropriately.\n\nAll options are forwarded to `Xandra.stream_pages!/4`, including\nretrying options.","ref":"Xandra.Cluster.html#stream_pages!/4"},{"type":"type","title":"Xandra.Cluster.cluster/0","doc":"A Xandra cluster.","ref":"Xandra.Cluster.html#t:cluster/0"},{"type":"type","title":"Xandra.Cluster.start_option/0","doc":"Cluster-specific options for `start_link/1`.\n\nSome of these options are internal and not part of the public API. Only use\nthe options explicitly documented in `start_link/1`.","ref":"Xandra.Cluster.html#t:start_option/0"},{"type":"module","title":"Xandra.Cluster.Host","doc":"The data structure to represent a host in a Cassandra cluster.\n\nThe fields of this struct are public. See `t:t/0` for information on their type,\nand [`%Xandra.Cluster.Host{}`](`__struct__/0`) for more information.","ref":"Xandra.Cluster.Host.html"},{"type":"function","title":"Xandra.Cluster.Host.__struct__/0","doc":"The struct that represents a host in a Cassandra cluster.\n\nSee `t:t/0` for the type of each field.","ref":"Xandra.Cluster.Host.html#__struct__/0"},{"type":"function","title":"Xandra.Cluster.Host.format_address/1","doc":"Formats a host's address and port as a string.","ref":"Xandra.Cluster.Host.html#format_address/1"},{"type":"function","title":"Examples - Xandra.Cluster.Host.format_address/1","doc":"iex> host = %Xandra.Cluster.Host{address: {127, 0, 0, 1}, port: 9042}\n    iex> Xandra.Cluster.Host.format_address(host)\n    \"127.0.0.1:9042\"\n\n    iex> host = %Xandra.Cluster.Host{address: \"example.com\", port: 9042}\n    iex> Xandra.Cluster.Host.format_address(host)\n    \"example.com:9042\"","ref":"Xandra.Cluster.Host.html#format_address/1-examples"},{"type":"type","title":"Xandra.Cluster.Host.t/0","doc":"The type for the host struct.","ref":"Xandra.Cluster.Host.html#t:t/0"},{"type":"type","title":"Fields - Xandra.Cluster.Host.t/0","doc":"* `:address` - the address of the host. It can be either an IP address\n    or a hostname. If Xandra managed to *connect* to this host, then the `:address` will\n    be the actual IP peer (see `:inet.peername/1`). Otherwise, the `:address` will be\n    the parsed IP or the charlist hostname. For example, if you pass `\"10.0.2.1\"` as\n    the address, Xandra will normalize it to `{10, 0, 2, 1}`.\n\n  * `:port` - the port of the host.\n\n  * `:data_center` - the data center of the host, as found in the `system.local` or\n    `system.peers` table.\n\n  * `:host_id` - the ID of the host, as found in the `system.local` or\n    `system.peers` table.\n\n  * `:rack` - the rack of the host, as found in the `system.local` or\n    `system.peers` table.\n\n  * `:release_version` - the release version of the host, as found in the `system.local` or\n    `system.peers` table.\n\n  * `:schema_version` - the schema version of the host, as found in the `system.local` or\n    `system.peers` table.\n\n  * `:tokens` - the tokens held by the host, as found in the `system.local` or\n    `system.peers` table.","ref":"Xandra.Cluster.Host.html#t:t/0-fields"},{"type":"behaviour","title":"Xandra.Cluster.LoadBalancingPolicy","doc":"A behaviour for defining load-balancing policies.\n\nA load-balancing policy is a way to tell Xandra which nodes to query when connected\nto a cluster.\n\nFor example, a simple load-balancing policy is `Xandra.Cluster.LoadBalancingPolicy.Random`,\nwhich picks a random node for each query out of the ones that are up.","ref":"Xandra.Cluster.LoadBalancingPolicy.html"},{"type":"callback","title":"Xandra.Cluster.LoadBalancingPolicy.host_added/2","doc":"Called when the Cassandra cluster reports a new host that joined.","ref":"Xandra.Cluster.LoadBalancingPolicy.html#c:host_added/2"},{"type":"callback","title":"Xandra.Cluster.LoadBalancingPolicy.host_connected/2","doc":"Called when Xandra successfully connects to `host`.","ref":"Xandra.Cluster.LoadBalancingPolicy.html#c:host_connected/2"},{"type":"callback","title":"Xandra.Cluster.LoadBalancingPolicy.host_down/2","doc":"Called when the Cassandra cluster marks `host` as \"down\".","ref":"Xandra.Cluster.LoadBalancingPolicy.html#c:host_down/2"},{"type":"callback","title":"Xandra.Cluster.LoadBalancingPolicy.host_removed/2","doc":"Called when the Cassandra cluster reports a host that left the cluster.","ref":"Xandra.Cluster.LoadBalancingPolicy.html#c:host_removed/2"},{"type":"callback","title":"Xandra.Cluster.LoadBalancingPolicy.host_up/2","doc":"Called when the Cassandra gossip marks `host` as \"up\".","ref":"Xandra.Cluster.LoadBalancingPolicy.html#c:host_up/2"},{"type":"callback","title":"Xandra.Cluster.LoadBalancingPolicy.hosts_plan/1","doc":"Called to return a \"plan\", which is an enumerable of hosts to start connections in order.","ref":"Xandra.Cluster.LoadBalancingPolicy.html#c:hosts_plan/1"},{"type":"callback","title":"Xandra.Cluster.LoadBalancingPolicy.init/1","doc":"Called to initialize the load-balancing policy.\n\n`options` is given by the user when configuring the cluster, and is specific to\nthe load-balancing policy.","ref":"Xandra.Cluster.LoadBalancingPolicy.html#c:init/1"},{"type":"callback","title":"Xandra.Cluster.LoadBalancingPolicy.query_plan/1","doc":"Called to return a \"plan\", which is an enumerable of hosts to query in order.","ref":"Xandra.Cluster.LoadBalancingPolicy.html#c:query_plan/1"},{"type":"type","title":"Xandra.Cluster.LoadBalancingPolicy.state/0","doc":"The state of the load-balancing policy.\n\nCan be any term and is passed around to all callbacks.","ref":"Xandra.Cluster.LoadBalancingPolicy.html#t:state/0"},{"type":"module","title":"Xandra.Cluster.LoadBalancingPolicy.DCAwareRoundRobin","doc":"A `Xandra.Cluster.LoadBalancingPolicy` that prefers hosts in a \"local\" data center.\n\nThis policy uses a **round-robin strategy** to pick hosts, giving precedence to hosts\nin a \"local\" data center. The local data center is determined by the\n`:local_data_center` option (see below). \"Giving precedence\" means that\n`hosts_plan/1` and `query_plan/1` will return a list of hosts where first there are all\nthe local hosts that are *up*, and then all the remote hosts that are *up*.\n\nThe round-robin strategy is applied to local and remote hosts separately. For example,\nsay the local hosts are `LH1`, `LH2`, `LH3`, and the remote hosts are `RH1`, `RH2`, `RH3`.\nThe first time, this policy will return `[LH1, RH1, LH2, RH2, LH3, RH3]`. The second time,\nit will return `[LH2, LH3, LH1, RH2, RH3, RH1]`. And so on.\n\nThis policy is available since Xandra *v0.15.0*.","ref":"Xandra.Cluster.LoadBalancingPolicy.DCAwareRoundRobin.html"},{"type":"module","title":"Options - Xandra.Cluster.LoadBalancingPolicy.DCAwareRoundRobin","doc":"This policy supports the following initialization options:\n\n* `:local_data_center` - The local data center. If `:from_first_peer`, this policy picks the local\n  data center from the first peer that is added to the cluster. This is\n  usually the first peer that the control connection connects to, which\n  (if everything goes well) is the first peer listed in the `:nodes`\n  option passed to `Xandra.Cluster.start_link/1`. If you want to force\n  a specific data center to be used as the local data center, you can\n  pass a string as the value of this option (such as `\"datacenter1\"`). The default value is `:from_first_peer`.","ref":"Xandra.Cluster.LoadBalancingPolicy.DCAwareRoundRobin.html#module-options"},{"type":"module","title":"Xandra.Cluster.LoadBalancingPolicy.Random","doc":"A simple `Xandra.Cluster.LoadBalancingPolicy` that picks hosts at random.\n\nThis load-balancing policy doesn't make any attempt to be smart: it doesn't take\ndata center or tokens into consideration, and considers all available nodes.\n\nThis policy is available since Xandra *v0.15.0*.","ref":"Xandra.Cluster.LoadBalancingPolicy.Random.html"},{"type":"behaviour","title":"Xandra.Compressor","doc":"A behaviour to compress and decompress binary data.\n\nModules implementing this behaviour can be used to compress and decompress\ndata using one of the compression algorithms supported by Cassandra (see below).","ref":"Xandra.Compressor.html"},{"type":"behaviour","title":"Supported algorithms and implementations - Xandra.Compressor","doc":"Native protocol versions v4 and earlier support two compression algorithms:\n[LZ4](https://en.wikipedia.org/wiki/LZ4_(compression_algorithm)) and\n[Snappy](https://en.wikipedia.org/wiki/Snappy_(compression)).\n\nNative protocol versions v5 and later only support LZ4.\n\n#","ref":"Xandra.Compressor.html#module-supported-algorithms-and-implementations"},{"type":"behaviour","title":"LZ4 - Xandra.Compressor","doc":"If you implement a compressor module for the LZ4 algorithm, then:\n\n  * The `c:compress/1` callback **must** return the compressed payload\n    *preceded* by a 32-bit big-endian unsigned integer representing the\n    length (in bytes) of the **uncompressed body**.\n\n  * Xandra will call the `c:decompress/1` callback with the compressed\n    payload, also preceded by the uncompressed body size (in bytes) as\n    a 32-bit big-endian unsigned integer.\n\nThat is, the result of compression when using the LZ4 algorithm must look like\nthis:\n\n     >\n\nSnappy is self-sufficient so it doesn't need the prepended uncompressed-payload\nsize.","ref":"Xandra.Compressor.html#module-lz4"},{"type":"behaviour","title":"Example - Xandra.Compressor","doc":"Let's imagine you implemented the LZ compression algorithm in your application:\n\n    defmodule MyApp.LZ4 do\n      def compress(binary), do: # ...\n      def decompress(binary, uncompressed_size), do: # ...\n    end\n\nYou can then implement a module that implements the `Xandra.Compressor`\nbehaviour and can be used to compress and decompress data flowing through the\nconnection to Cassandra:\n\n    defmodule LZ4XandraCompressor do\n      @behaviour Xandra.Compressor\n\n      @impl true\n      def algorithm(), do: :lz4\n\n      @impl true\n      def compress(body) do\n        [ >, MyApp.LZ4.compress(body)]\n      end\n\n      @impl true\n      def decompress( >) do\n        MyApp.LZ4.decompress(compressed_body, uncompressed_size)\n      end\n    end\n\nNow, this module can be passed as the value of the `:compressor` option to\nmany functions in `Xandra`:\n\n    Xandra.start_link(compressor: LZ4XandraCompressor)\n\nFor more information on compression, see the \"Compression\" section in the\ndocumentation for `Xandra.`","ref":"Xandra.Compressor.html#module-example"},{"type":"callback","title":"Xandra.Compressor.algorithm/0","doc":"Specifies which algorithm this module will use to compress and decompress\ndata.","ref":"Xandra.Compressor.html#c:algorithm/0"},{"type":"callback","title":"Xandra.Compressor.compress/1","doc":"Compresses the given iodata according to the algorithm returned by\n`c:algorithm/0`.","ref":"Xandra.Compressor.html#c:compress/1"},{"type":"callback","title":"Xandra.Compressor.decompress/1","doc":"Decompresses the given binary according to the algorithm returned by\n`c:algorithm/0`.","ref":"Xandra.Compressor.html#c:decompress/1"},{"type":"module","title":"Xandra.Page","doc":"A struct that represents a page of rows.\n\nThis struct represents a page of rows that have been returned by the\nCassandra server in response to a query such as `SELECT`, but have not yet\nbeen parsed into Elixir values.\n\nThis struct implements the `Enumerable` protocol and is therefore a stream. It\nis through this protocol that a `Xandra.Page` struct can be parsed into Elixir\nvalues. The simplest way of getting a list of single rows out of a\n`Xandra.Page` struct is to use something like `Enum.to_list/1`. Each element\nemitted when streaming out of a `Xandra.Page` struct is a map of string column\nnames to their corresponding value.\n\nSee [`%Xandra.Page{}`](`__struct__/0`) for information about which fields are public.","ref":"Xandra.Page.html"},{"type":"module","title":"Examples - Xandra.Page","doc":"statement = \"SELECT name, age FROM users\"\n\n    %Xandra.Page{} = page = Xandra.execute!(conn, statement, _params = [])\n\n    Enum.each(page, fn %{\"name\" => name, \"age\" => age} ->\n      IO.puts(\"Read user with name #{name} (age #{age}) out of the database\")\n    end)","ref":"Xandra.Page.html#module-examples"},{"type":"function","title":"Xandra.Page.__struct__/0","doc":"The page struct.\n\nThe following fields are public and can be accessed or relied on:\n\n  * `:paging_state` - the current paging state, or `nil` if no paging is occurring.\n    Its value can be used to check whether more pages are available to fetch\n    after the given page. This is useful when implementing manual paging.\n    See also the documentation for `Xandra.execute/4`.\n\n  * `:tracing_id` - the tracing ID (as a UUID binary) if tracing was enabled,\n    or `nil` if no tracing was enabled. See the \"Tracing\" section in `Xandra.execute/4`.","ref":"Xandra.Page.html#__struct__/0"},{"type":"type","title":"Xandra.Page.paging_state/0","doc":"The paging state of a page.\n\nThis is intended to be an \"opaque\" binary value that you can use for further\npagination. See `Xandra.execute/4`.","ref":"Xandra.Page.html#t:paging_state/0"},{"type":"type","title":"Xandra.Page.t/0","doc":"The type for the page struct.\n\nThe only public fields here are `:paging_state` and `:tracing_id`.\nSee [`%Xandra.Page{}`](`__struct__/0`).","ref":"Xandra.Page.html#t:t/0"},{"type":"module","title":"Xandra.Prepared","doc":"A data structure used to internally represent prepared queries.\n\nSee [`%Xandra.Prepared{}`](`__struct__/0`) for information about which fields are\npublic. All other fields are documented in `t:t/0` to avoid Dialyzer warnings,\nbut are not meant to be used by users.","ref":"Xandra.Prepared.html"},{"type":"function","title":"Xandra.Prepared.__struct__/0","doc":"A struct that represents a prepared query.\n\nThese are the publicly-accessible fields of this struct:\n\n  * `:tracing_id` - the tracing ID (as a UUID binary) if tracing was enabled,\n    or `nil` if no tracing was enabled. See the \"Tracing\" section in `Xandra.execute/4`.\n\n  * `:response_custom_payload` - the *custom payload* sent by the server, if present.\n    If the server doesn't send a custom payload, this field is `nil`. Otherwise,\n    it's of type `t:Xandra.custom_payload/0`. See the \"Custom payloads\" section\n    in the documentation for the `Xandra` module.","ref":"Xandra.Prepared.html#__struct__/0"},{"type":"type","title":"Xandra.Prepared.t/0","doc":"The type for a prepared query.\n\nThe only public fields here are `:tracing_id` and `:response_custom_payload`.\nSee [`%Xandra.Prepared{}`](`__struct__/0`).","ref":"Xandra.Prepared.html#t:t/0"},{"type":"behaviour","title":"Xandra.RetryStrategy","doc":"A behaviour that handles how to retry failed queries.\n\nThis behaviour makes it possible to customize the strategy that Xandra uses to\n*retry failed queries*. By default, Xandra does not retry failed queries, and\ndoes not provide any default retry strategy since retrying queries based on\nthe failure reason is very tied to application logic.\n\n> ##","ref":"Xandra.RetryStrategy.html"},{"type":"behaviour","title":"Why Do You Need This? {: .info} - Xandra.RetryStrategy","doc":">\n> You might be wondering why there's a need for a retry strategy behaviour, when\n> you could do this at the application layer by simply checking the return value\n> of Xandra calls and potentially retrying.\n>\n> Well, the reason is that retrying queries to Cassandra can get quite smart. For\n> example, you might want context to know what Cassandra node a query failed to\n> execute on, so that you can try it *on a different node* next. `Xandra.RetryStrategy`\n> modules get all the necessary info to implement this kind of smarter strategies.\n\nA module that implements the `Xandra.RetryStrategy` behaviour can be passed to\nseveral functions in the `Xandra` module and `Xandra.Cluster` modules. look at\nthe documentation for those modules for more information.","ref":"Xandra.RetryStrategy.html#module-why-do-you-need-this-info"},{"type":"behaviour","title":"Usage - Xandra.RetryStrategy","doc":"When a query fails and a retry strategy module was passed as an option, Xandra\nwill:\n\n  1. **Invoke the `c:new/1` callback** — it will invoke this with the options\n     passed to the failing function to initialize the given retry strategy.\n     This gives you access to things like default consistency, timeouts, and\n     so on.\n\n  1. **Invoke the `c:retry/3` callback until necessary** — Xandra will ask the\n     retry strategy whether to retry or error out until either\n     the query succeeds or `c:retry/3` says to error out.\n\n> ##","ref":"Xandra.RetryStrategy.html#module-usage"},{"type":"behaviour","title":"Process {: .neutral} - Xandra.RetryStrategy","doc":">\n> The `c:new/1` and `c:retry/3` callbacks will be invoked in the same\n> process that executed the original query.\n\n#","ref":"Xandra.RetryStrategy.html#module-process-neutral"},{"type":"behaviour","title":"Single Connections or Clusters - Xandra.RetryStrategy","doc":"There are two possible cases where a retry strategy is invoked: either it's invoked\nwhen a query fails to execute on a single connection (that is, it was executed through\na `Xandra` function), or when a query fails to execute through a cluster connection (that\nis, it was executed through a `Xandra.Cluster` function).\n\nTo distinguish these cases, Xandra always passes the `:execution_level` option\nto `c:new/1`. This option has the type `t:execution_level/0`.\n\nIf the execution level is `:single_connection`, Xandra doesn't inject any additional\noptions. When the execution level is `:single_connection`, `c:retry/3` can only return\nthe 3-element version of the `{:retry, ...}` tuple.\n\nIf the execution level is `:cluster`, Xandra injects these options when calling `c:new/1`:\n\n  * `:connected_hosts` — a list of `{connection_pid, host}` tuples, where\n    `connection_pid` (a `t:pid/0`) is the PID of the connection and `host` (a\n    `t:Xandra.Cluster.Host.t/0`) is the corresponding host information. You can use\n    this option to determine on which node to retry a query. Elements in this list\n    are ordered according to the `Xandra.Cluster.LoadBalancingPolicy` used by the\n    cluster. If you want to keep track of the original `:connected_hosts`, you'll\n    need to store them in the state of the retry strategy returned by `c:new/1`.\n\nWhen the execution level is `:single_connection`, `c:retry/3` can only return\nthe 4-element version of the `{:retry, ...}` tuple.","ref":"Xandra.RetryStrategy.html#module-single-connections-or-clusters"},{"type":"behaviour","title":"Examples - Xandra.RetryStrategy","doc":"Let's look at some examples.\n\n#","ref":"Xandra.RetryStrategy.html#module-examples"},{"type":"behaviour","title":"Retry Count - Xandra.RetryStrategy","doc":"This is an example of a retry strategy that retries a fixed number of times\nbefore failing. It injects `:retry_count` option which it uses to keep track\nof how many times the query failed. This is effectively the `t:state/0` of this\nretry strategy.\n\n    defmodule MyApp.CounterRetryStrategy do\n      @behaviour Xandra.RetryStrategy\n\n      @impl true\n      def new(options) do\n        # This is the \"state\" of this retry strategy\n        Keyword.fetch!(options, :retry_count)\n      end\n\n      @impl true\n      def retry(error, options, retries_left)\n\n      def retry(_error, _options, _retries_left = 0) do\n        :error\n      end\n\n      def retry(_error, options, retries_left = _state) do\n        {:retry, options, retries_left - 1}\n      end\n    end\n\n#","ref":"Xandra.RetryStrategy.html#module-retry-count"},{"type":"behaviour","title":"Downgrading Consistency - Xandra.RetryStrategy","doc":"Another interesting example could be a retry strategy based on **downgrading\nconsistency**: for example, we could execute all queries with a \"high\" consistency\n(such as `:all`) at first, and in case of failure, try again with a lower\nconsistency (such as `:quorum`), finally giving up if that fails as well.\n\n    defmodule MyApp.DowngradingConsistencyRetryStrategy do\n      @behaviour Xandra.RetryStrategy\n\n      @impl true\n      def new(_options) do\n        :no_state\n      end\n\n      @impl true\n      def retry(_error, options, :no_state) do\n        case Keyword.fetch(options, :consistency) do\n          # No consistency was specified, so we don't bother to retry.\n          :error -> :error\n\n          # If the consistency was :all, we downgrade it by injecting a new one in the options.\n          {:ok, :all} -> {:retry, Keyword.replace!(options, :consistency, :quorum), :no_state}\n\n          # If the consistency was already lower than :all, we give up and stop retrying.\n          {:ok, _other} -> :error\n        end\n      end\n    end\n\n#","ref":"Xandra.RetryStrategy.html#module-downgrading-consistency"},{"type":"behaviour","title":"Different Node (for Clusters) - Xandra.RetryStrategy","doc":"A particularly-useful application of retry strategies is to retry queries on different hosts\nwhen using `Xandra.Cluster`. We can even choose not to execute on certain hosts\n(because they may be in a different data center). The following example retries on all hosts\nafter the first `:connected_node` has failed:\n\n    defmodule MyApp.AllNodesRetryStrategy do\n      @behaviour Xandra.RetryStrategy\n\n      alias Xandra.Cluster.Host\n\n      @impl true\n      def new(options) do\n        if options[:execution_level] != :cluster do\n          raise ArgumentError, \"this retry strategy can only be used with clusters\"\n        end\n\n        [_already_tried_node | remaining_nodes] = Keyword.fetch!(options, [:connected_hosts])\n        remaining_nodes\n      end\n\n      @impl true\n      def retry(error, options, nodes)\n\n      # No nodes left to retry on.\n      def retry(_error, options, [] = _remaining_nodes) do\n        :error\n      end\n\n      def retry(_error, options, [{conn_pid, %Host{}} | remaining_nodes]) do\n        {:retry, options, _new_state = remaining_nodes, conn_pid}\n      end\n    end","ref":"Xandra.RetryStrategy.html#module-different-node-for-clusters"},{"type":"callback","title":"Xandra.RetryStrategy.new/1","doc":"Initializes the state of a retry strategy based on the given `options`.","ref":"Xandra.RetryStrategy.html#c:new/1"},{"type":"callback","title":"Xandra.RetryStrategy.retry/3","doc":"Determines whether to retry the failed query or return the error to the caller.\n\nThe first argument is the **error** that caused the query to fail: for example, it\ncould be a `Xandra.Error` struct with reason `:read_timeout`. This can be used\nto potentially determine the retry strategy based on the failure reason. The second\nargument is the options given to the function that failed while executing the\nquery. The third argument is the retry strategy state returned either by\n`c:new/1` or by subsequent calls to `c:retry/3`.","ref":"Xandra.RetryStrategy.html#c:retry/3"},{"type":"callback","title":"Return Values - Xandra.RetryStrategy.retry/3","doc":"If `:error` is returned, the function that was trying to execute the query\nwill return the error to the caller instead of retrying.\n\nIf `{:retry, new_options, new_state}` is returned, the function that was\ntrying to execute the query will be invoked again with the same query and\n`new_options` as its options. `new_state` will be used if the query fails\nagain: in that case, `c:retry/3` will be invoked again with `new_state` as its\nthird argument. This sequence of steps will repeat until either the query is executed\nsuccessfully or this callback returns `:error`.\n\nThe last possible return value is `{:retry, new_options, new_state, conn_pid}`.\nThis can only be returned by retry strategies used by `Xandra.Cluster`, and any\nattempt to return this when using `Xandra` function will result in an error. This\nreturn value is *available since v0.18.0*.","ref":"Xandra.RetryStrategy.html#c:retry/3-return-values"},{"type":"type","title":"Xandra.RetryStrategy.execution_level/0","doc":"The possible values of the `:execution_level` option injected into the options\npassed to `c:new/1`.","ref":"Xandra.RetryStrategy.html#t:execution_level/0"},{"type":"type","title":"Xandra.RetryStrategy.state/0","doc":"The state of the retry strategy.","ref":"Xandra.RetryStrategy.html#t:state/0"},{"type":"module","title":"Xandra.SchemaChange","doc":"A struct that represents the result of a query that modifies the schema.\n\nSee [`%Xandra.SchemaChange{}`](`__struct__/0`) for information on the fields of this struct.","ref":"Xandra.SchemaChange.html"},{"type":"function","title":"Xandra.SchemaChange.__struct__/0","doc":"The struct for the \"schema change\" result.\n\nThis struct has the following fields:\n\n  * `:effect` - the type of change involved. It's one of `\"CREATED\"`,\n    `\"UPDATED\"`, or `\"DROPPED\"`.\n\n  * `:target` - what has been modified. It's one of `\"KEYSPACE\"`, `\"TABLE\"`,\n    or `\"TYPE\"`.\n\n  * `:options` - a map of options that depends on the value of `:target`:\n    * if target is `\"KEYSPACE\"`, the map will have the form\n      `%{keyspace: keyspace}`\n    * if the target is `\"TABLE\"` or `\"TYPE\"`, the map will have the form\n      `%{keyspace: keyspace, subject: subject}` where `keyspace` is the\n      keyspace where the change happened and `subject` is the name of what\n      changed (so the name of the changed table or type)\n\n  * `:tracing_id` - the tracing ID (as a UUID binary) if tracing was enabled,\n    or `nil` if no tracing was enabled. See the \"Tracing\" section in `Xandra.execute/4`.\n\n  * `:custom_payload` - the *custom payload* sent by the server, if present.\n    If the server doesn't send a custom payload, this field is `nil`. Otherwise,\n    it's of type `t:Xandra.custom_payload/0`. See the \"Custom payloads\" section\n    in the documentation for the `Xandra` module.","ref":"Xandra.SchemaChange.html#__struct__/0"},{"type":"type","title":"Xandra.SchemaChange.t/0","doc":"The type for a schema change result.\n\nSee [`%Xandra.SchemaChange{}`](`__struct__/0`) for information.","ref":"Xandra.SchemaChange.html#t:t/0"},{"type":"module","title":"Xandra.SetKeyspace","doc":"A struct that represents the result of a `USE` query.\n\nThese are the public fields of this struct:\n\n  * `:keyspace` - the keyspace (as a binary) that was set through the executed\n    `USE` query.\n\n  * `:tracing_id` - the tracing ID (as a UUID binary) if tracing was enabled,\n    or `nil` if no tracing was enabled. See the \"Tracing\" section in `Xandra.execute/4`.\n\n  * `:custom_payload` - the *custom payload* sent by the server, if present.\n    If the server doesn't send a custom payload, this field is `nil`. Otherwise,\n    it's of type `t:Xandra.custom_payload/0`. See the \"Custom payloads\" section\n    in the documentation for the `Xandra` module.","ref":"Xandra.SetKeyspace.html"},{"type":"type","title":"Xandra.SetKeyspace.t/0","doc":"The type for a \"set keyspace\" result.","ref":"Xandra.SetKeyspace.html#t:t/0"},{"type":"module","title":"Xandra.Simple","doc":"A data structure used internally to represent simple queries.","ref":"Xandra.Simple.html"},{"type":"type","title":"Xandra.Simple.t/0","doc":"The type for a simple query.\n\nThe fields of this are not meant to be used, but are documented to avoid\nDialyzer warnings.","ref":"Xandra.Simple.html#t:t/0"},{"type":"module","title":"Xandra.Telemetry","doc":"Telemetry integration for event tracing, metrics, and logging.\n\nXandra uses [telemetry](https://github.com/beam-telemetry/telemetry) for reporting\nmetrics and events. Below we list all the possible events emitted by Xandra, alongside\ntheir measurements and metadata.\n\nXandra emits telemetry events *since v0.15.0*.","ref":"Xandra.Telemetry.html"},{"type":"module","title":"Events - Xandra.Telemetry","doc":"For a comprehensive list of the events that Xandra emits, see the\n[*Telemetry Events*](telemetry-events.html) page in the guides.","ref":"Xandra.Telemetry.html#module-events"},{"type":"function","title":"Xandra.Telemetry.attach_default_handler/0","doc":"Attaches a handler that **logs** Telemetry events.\n\nThis handler is useful when you want to see what's going on in Xandra without having to write a\nTelemetry handler to handle all the events.\n\nThese are the events that get logged. This list might change in the future.\n\n| **Event**                                                      | **Level** |\n| -------------------------------------------------------------- | --------- |\n| `[:xandra, :connected]`                                        | info      |\n| `[:xandra, :disconnected]`                                     | warn      |\n| `[:xandra, :failed_to_connect]`                                | warn      |\n| `[:xandra, :prepared_cache, :hit]`                             | debug     |\n| `[:xandra, :prepared_cache, :miss]`                            | debug     |\n| `[:xandra, :prepare_query, :start]`                            | debug     |\n| `[:xandra, :prepare_query, :stop]`                             | debug     |\n| `[:xandra, :prepare_query, :exception]`                        | error     |\n| `[:xandra, :execute_query, :start]`                            | debug     |\n| `[:xandra, :execute_query, :stop]`                             | debug     |\n| `[:xandra, :execute_query, :exception]`                        | error     |\n| `[:xandra, :server_warnings]`                                  | warn      |\n| `[:xandra, :cluster, :change_event]`                           | debug     |\n| `[:xandra, :cluster, :control_connection, :connected]`         | debug     |\n| `[:xandra, :cluster, :control_connection, :disconnected]`      | debug     |\n| `[:xandra, :cluster, :control_connection, :failed_to_connect]` | warn      |\n| `[:xandra, :cluster, :pool, :started]`                         | debug     |\n| `[:xandra, :cluster, :pool, :restarted]`                       | debug     |\n| `[:xandra, :cluster, :discovered_peers]`                       | debug     |\n\nEvents have the following logger metadata:\n\n  * `:xandra_address` - the address of the node the connection is connected to\n  * `:xandra_port` - the port of the node the connection is connected to\n  * `:xandra_protocol_module` - the protocol module for the Cassandra native protocol","ref":"Xandra.Telemetry.html#attach_default_handler/0"},{"type":"module","title":"Xandra.Void","doc":"A struct that represents an empty Cassandra result.\n\nThis struct is returned as the result of queries such as `INSERT`, `UPDATE`, or\n`DELETE`. See [`%Xandra.Void{}`](`__struct__/0`) for information about the fields.","ref":"Xandra.Void.html"},{"type":"function","title":"Xandra.Void.__struct__/0","doc":"The struct for \"void\" results.\n\nThese are the public fields it contains:\n\n  * `:tracing_id` - the tracing ID (as a UUID binary) if tracing was enabled,\n    or `nil` if no tracing was enabled. See the \"Tracing\" section in `Xandra.execute/4`.\n\n  * `:custom_payload` - the *custom payload* sent by the server, if present.\n    If the server doesn't send a custom payload, this field is `nil`. Otherwise,\n    it's of type `t:Xandra.custom_payload/0`. See the \"Custom payloads\" section\n    in the documentation for the `Xandra` module.","ref":"Xandra.Void.html#__struct__/0"},{"type":"type","title":"Xandra.Void.t/0","doc":"The type for a \"void\" result.","ref":"Xandra.Void.html#t:t/0"},{"type":"exception","title":"Xandra.ConnectionError","doc":"An exception struct that represents an error in the connection to the\nCassandra server.\n\nFor more information on when this error is returned or raised, see the\ndocumentation for the `Xandra` module.\n\nThe `:action` field represents the action that was being performed when the\nconnection error occurred. The `:reason` field represents the reason of the\nconnection error: for network errors, this is usually a POSIX reason (like\n`:econnrefused`). The following Xandra-specific reasons are supported:\n\n  * `{:unsupported_compression, algorithm}` - this happens when a\n    `:compressor` module has been specified in `Xandra.start_link/1`, but\n    negotiating the connection algorithm fails because such compressor module\n    uses an algorithm that the Cassandra server does not support.\n\n  * `:disconnected` - the connection closed in the middle of a request to the server.\n\n  * `{:connection_process_crashed, reason}` - the connection process crashed before\n    sending a response.\n\n  * `:timeout` - the connection timed out while waiting for a response from the\n    server.\n\n  * `{:cluster, :not_connected}` - this happens when a `Xandra.Cluster`-based\n    connection is not connected to any node (for example, because all the\n    specified nodes are currently down). See the documentation for\n    `Xandra.Cluster` for more information.\n\nSince this struct is an exception, it is possible to raise it with\n`Kernel.raise/1`. If the intent is to format connection errors as strings (for\nexample, for logging purposes), it is possible to use `Exception.message/1` to\nget a formatted version of the error.","ref":"Xandra.ConnectionError.html"},{"type":"function","title":"Xandra.ConnectionError.new/2","doc":"","ref":"Xandra.ConnectionError.html#new/2"},{"type":"type","title":"Xandra.ConnectionError.t/0","doc":"","ref":"Xandra.ConnectionError.html#t:t/0"},{"type":"exception","title":"Xandra.Error","doc":"An exception struct that represents an error returned by Cassandra.\n\nFor more information on when this error is returned or raised, see the\ndocumentation for the `Xandra` module.\n\nThe `:reason` field represents the reason (as an atom) of the error. For\nexample, if the query you're trying to execute contains a syntax error,\n`:reason` will be `:invalid_syntax`. The `:message` field is a string that\ncontains the exact error message that Cassandra returned.\n\nSince this struct is an exception, it is possible to raise it with\n`Kernel.raise/1`. If the intent is to format errors as strings (for\nexample, for logging purposes), it is possible to use `Exception.message/1` to\nget a formatted version of the error.","ref":"Xandra.Error.html"},{"type":"function","title":"Xandra.Error.__struct__/0","doc":"The exception struct for a Cassandra error.","ref":"Xandra.Error.html#__struct__/0"},{"type":"type","title":"Xandra.Error.t/0","doc":"The type for a Cassandra error exception.","ref":"Xandra.Error.html#t:t/0"},{"type":"extras","title":"Data Types Comparison Table","doc":"# Data Types Comparison Table\n\nThis page contains a table that compares the data types used by Cassandra with their counterparts used by Xandra. Each Cassandra data type corresponds to one or more Elixir data types. What Elixir data type is used for a given Cassandra type is often controlled by a formatting option. For example, a `date` value from Cassandra can be returned as a `Date.t()` struct or as an integer representing the number of days since the Unix epoch depending on the `:date_format` option passed to various Xandra functions.","ref":"data-types-comparison-table.html"},{"type":"extras","title":"Comparison Table - Data Types Comparison Table","doc":"| Cassandra data type | Elixir data type                                                                 |\n| ------------------- | -------------------------------------------------------------------------------- |\n| `ascii`             | `t:String.t/0`                                                                   |\n| `bigint`            | `t:integer/0`                                                                    |\n| `blob`              | `t:binary/0`                                                                     |\n| `boolean`           | `t:boolean/0`                                                                    |\n| `counter`           | `t:integer/0`                                                                    |\n| `date`              | `t:Date.t/0` (if `date_format: :date`)                                           |\n| `date`              | `t:integer/0` (if `date_format: :integer`), days from Unix epoch                 |\n| `decimal`           | `{value, scale}` (if `decimal_format: :tuple`), where `value * 10^(-1 * scale)`  |\n| `decimal`           | `t:Decimal.t/0` (if `decimal_format: :decimal`)                                  |\n| `double`            | `t:float/0`                                                                      |\n| `float`             | `t:float/0`                                                                      |\n| `inet`              | `t::inet.ip_address/0`                                                           |\n| `int`               | `t:integer/0`                                                                    |\n| `list `           | `t:list/0`                                                                       |\n| `map ` | `t:map/0`                                                                        |\n| `NULL`              | `nil`                                                                            |\n| `set `            | `t:MapSet.t/0`                                                                   |\n| `smallint`          | `t:integer/0`                                                                    |\n| `text`              | `t:String.t/0`                                                                   |\n| `time`              | `t:integer/0` (if `time_format: :integer`), as nanoseconds from midnight         |\n| `time`              | `t:Time.t/0` (if `time_format: :time`)                                           |\n| `timestamp`         | `t:DateTime.t/0` (if `timestamp_format: :datetime`)                              |\n| `timestamp`         | `t:integer/0` (if `timestamp_format: :integer`), as milliseconds from Unix epoch |\n| `timeuuid`          | `t:binary/0`                                                                     |\n| `tinyint`           | `t:integer/0`                                                                    |\n| `tuple `        | `t:tuple/0`                                                                      |\n| `uuid`              | `t:binary/0`                                                                     |\n| `varchar`           | `t:String.t/0`                                                                   |\n| `varint`            | `t:integer/0`                                                                    |","ref":"data-types-comparison-table.html#comparison-table"},{"type":"extras","title":"Compatibility","doc":"# Compatibility\n\nThis page describes Xandra's compatibility with various Cassandra versions,\nCassandra-compatible data storage (like ScyllaDB), and Cassandra native protocol\nversions.\n\nAll compatibility below refers to the latest version of Xandra.","ref":"compatibility.html"},{"type":"extras","title":"Cassandra Versions - Compatibility","doc":"* `3.x`\n  * `4.x`","ref":"compatibility.html#cassandra-versions"},{"type":"extras","title":"Native Protocol Versions - Compatibility","doc":"* `v3`\n  * `v4`\n  * `v5`","ref":"compatibility.html#native-protocol-versions"},{"type":"extras","title":"ScyllaDB Versions - Compatibility","doc":"* `2.3`\n  * `3.x`\n  * `4.x`\n  * `5.x`","ref":"compatibility.html#scylladb-versions"},{"type":"extras","title":"Telemetry Events","doc":"# Telemetry Events","ref":"telemetry-events.html"},{"type":"extras","title":"Connection Events - Telemetry Events","doc":"### `[:xandra, :connected]`\n\n\n\nExecuted when a connection connects to its Cassandra node.\n\n\n**Measurements**: *none*\n\n\n\n**Metadata**:\n\n* `:connection` (`t:pid/0`) - the PID of the connection process\n* `:connection_name` (`t:term/0` or `nil`) - given name of the connection or `nil` if not set\n* `:address` (`t:String.t/0`) - the address of the node the connection is connected to\n* `:port` (`t::inet.port_number/0`) - the port of the node the connection is connected to\n* `:protocol_module` (`t:module/0`) - the protocol module used for the connection\n* `:supported_options` (`t:map/0`) - Cassandra supported options (mostly useful for internal debugging)\n\n\n\n\n### `[:xandra, :disconnected]`\n\n\n\nExecuted when a connection disconnects from its Cassandra node.\n\n\n**Measurements**: *none*\n\n\n\n**Metadata**:\n\n* `:connection` (`t:pid/0`) - the PID of the connection process\n* `:connection_name` (`t:term/0` or `nil`) - given name of the connection or `nil` if not set\n* `:address` (`t:String.t/0`) - the address of the node the connection is connected to\n* `:port` (`t::inet.port_number/0`) - the port of the node the connection is connected to\n* `:reason` (usually a `t:Exception.t/0`) - the reason for the disconnection\n\n\n\n\n### `[:xandra, :failed_to_connect]`\n\n\n*Available since v0.18.0*.\n\n\nExecuted when a connection fails to connect to its Cassandra node.\n\n\n**Measurements**: *none*\n\n\n\n**Metadata**:\n\n* `:connection` (`t:pid/0`) - the PID of the connection process\n* `:connection_name` (`t:term/0` or `nil`) - given name of the connection or `nil` if not set\n* `:address` (`t:String.t/0`) - the address of the node the connection is connected to\n* `:port` (`t::inet.port_number/0`) - the port of the node the connection is connected to\n* `:reason` (usually a `t:Exception.t/0`) - the reason for the disconnection","ref":"telemetry-events.html#connection-events"},{"type":"extras","title":"Query Events - Telemetry Events","doc":"The `[:xandra, :prepare_query, ...]` and `[:xandra, :execute_query, ...]` events are\nTelemetry **spans**. See\n[`telemetry:span/3`](https://hexdocs.pm/telemetry/telemetry.html#span/3). All the time\nmeasurements are in *native* time unit, so you need to use `System.convert_time_unit/3`\nto convert to the desired time unit.\n\n\n\n### `[:xandra, :prepare_query, ...]`\n\n\n\nExecuted before and after a query is prepared (as a Telemetry **span**).\n\n\n**Measurements**:\n\n* `:system_time` (`t:integer/0`) - in `:native` time units (only for `[..., :start]` events)\n* `:monotonic_time` (`t:integer/0`) - in `:native` time units\n* `:duration` (`t:integer/0`) - in `:native` time units (only for `[..., :stop]` and `[..., :exception]` events)\n\n\n\n\n**Metadata**:\n\n* `:connection` (`t:pid/0`) - the PID of the connection process\n* `:connection_name` (`t:term/0` or `nil`) - given name of the connection or `nil` if not set\n* `:address` (`t:String.t/0`) - the address of the node the connection is connected to\n* `:port` (`t::inet.port_number/0`) - the port of the node the connection is connected to\n* `:query` (`t:Xandra.Prepared.t/0`) - the query being prepared\n* `:extra_metadata` (any term) - extra metadata provided by the `:telemetry_metadata` option\n* `:reprepared` (`t:boolean/0`) - whether the query was reprepared or not (only available for\n`[..., :stop]` events)\n* `:reason` (any term) - if there the result of the query was an error, this is the reason\n(only available for `[..., :stop]` events), otherwise it's the error\nthat was raised (only available for `[..., :exception]` events)\n* `:kind` (`t:Exception.kind/0`) - exception kind (only available for `[..., :exception]` events)\n* `:stacktrace` (`t:Exception.stacktrace/0`) - exception stacktrace (only available for `[..., :exception]` events)\n\n\n\n\n### `[:xandra, :execute_query, ...]`\n\n\n\nExecuted before and after a query is executed (as a Telemetry **span**).\n\n\n**Measurements**:\n\n* `:system_time` (`t:integer/0`) - in `:native` time units (only for `[..., :start]` events)\n* `:monotonic_time` (`t:integer/0`) - in `:native` time units\n* `:duration` (`t:integer/0`) - in `:native` time units (only for `[..., :stop]` and `[..., :exception]` events)\n\n\n\n\n**Metadata**:\n\n* `:connection` (`t:pid/0`) - the PID of the connection process\n* `:connection_name` (`t:term/0` or `nil`) - given name of the connection or `nil` if not set\n* `:address` (`t:String.t/0`) - the address of the node the connection is connected to\n* `:port` (`t::inet.port_number/0`) - the port of the node the connection is connected to\n* `:query` (`t:Xandra.Simple.t/0`, `t:Xandra.Batch.t/0`, or `t:Xandra.Prepared.t/0`) - the query being executed\n* `:extra_metadata` (any term) - extra metadata provided by the `:telemetry_metadata` option\n* `:reason` (any term) - if there the result of the query was an error, this is the reason\n(only available for `[..., :stop]` events), otherwise it's the error\nthat was raised (only available for `[..., :exception]` events)\n* `:kind` (`t:Exception.kind/0`) - exception kind (only available for `[..., :exception]` events)\n* `:stacktrace` (`t:Exception.stacktrace/0`) - exception stacktrace (only available for `[..., :exception]` events)\n\n\n\n\n### `[:xandra, :prepared_cache, :hit | :miss]`\n\n\n\nExecuted when a query is executed and the prepared cache is checked.\n\n\n**Measurements**: *none*\n\n\n\n**Metadata**:\n\n* `:connection` (`t:pid/0`) - the PID of the connection process\n* `:connection_name` (`t:term/0` or `nil`) - given name of the connection or `nil` if not set\n* `:address` (`t:String.t/0`) - the address of the node the connection is connected to\n* `:port` (`t::inet.port_number/0`) - the port of the node the connection is connected to\n* `:query` (`t:Xandra.Prepared.t/0`) - the query being prepared\n* `:extra_metadata` (any term) - extra metadata provided by the `:telemetry_metadata` option","ref":"telemetry-events.html#query-events"},{"type":"extras","title":"Warnings - Telemetry Events","doc":"### `[:xandra, :server_warnings]`\n\n\n\nExecuted when a query returns warnings.\n\n\n**Measurements**:\n\n* `:warnings` (non-empty list of `t:String.t/0`) - a list of warnings\n\n\n\n\n**Metadata**:\n\n* `:address` (`t:String.t/0`) - the address of the node the connection is connected to\n* `:port` (`t::inet.port_number/0`) - the port of the node the connection is connected to\n* `:current_keyspace` (`t:String.t/0` or `nil`) - the current keyspace of the connection, or `nil` if not set\n* `:query` (`t:Xandra.Simple.t/0`, `t:Xandra.Batch.t/0`, or `t:Xandra.Prepared.t/0`) - the query that caused the warnings","ref":"telemetry-events.html#warnings"},{"type":"extras","title":"Cluster Events - Telemetry Events","doc":"Unless specified otherwise, these are available since v0.15.0.\n\n\n### `[:xandra, :cluster, :change_event]`\n\n\n\nEmitted when there is a change in the cluster, either as reported by Cassandra itself\nor as detected by Xandra.\n\n\n\n**Measurements**: *none*\n\n\n\n**Metadata**:\n\n* `:cluster_pid` (`t:pid/0`) - the PID of the cluster process\n* `:cluster_name` (`t:String.t/0` or `nil`) - the name of the cluster executing the event, if provided\nthrough the `:name` option in `Xandra.Cluster.start_link/1`\n* `:host` (`t:Xandra.Cluster.Host.t/0`) - the host the event is related to\n* `:event_type` (`t:atom/0`) - one of `:host_up` (a host went up), `:host_down` (a host went down),\n`:host_added` (a host was added to the cluster topology), or `:host_removed`\n(a host was removed from the cluster topology)\n\n\n\n\n### `[:xandra, :cluster, :discovered_peers]`\n\n\n*Available since v0.17.0*.\n\n\nExecuted when the Xandra cluster's control connection discovers peers. The peers might have been\nalready discovered in the past, so you'll need to keep track of new peers if you need to.\n\n\n\n**Measurements**:\n\n* `:peers` (list of `t:Xandra.Cluster.Host.t/0`) - the discovered peers\n\n\n\n\n**Metadata**:\n\n* `:cluster_pid` (`t:pid/0`) - the PID of the cluster process\n* `:cluster_name` (`t:String.t/0` or `nil`) - the name of the cluster executing the event, if provided\nthrough the `:name` option in `Xandra.Cluster.start_link/1`\n\n\n\n\n### `[:xandra, :cluster, :pool, :started | :restarted | :stopped]`\n\n\n*Available since v0.17.0*.\n\n\nExecuted when a connection pool to a node is started, restarted, or stopped.\n\n\n\n**Measurements**: *none*\n\n\n\n**Metadata**:\n\n* `:cluster_pid` (`t:pid/0`) - the PID of the cluster process\n* `:cluster_name` (`t:String.t/0` or `nil`) - the name of the cluster executing the event, if provided\nthrough the `:name` option in `Xandra.Cluster.start_link/1`\n* `:host` (`t:Xandra.Cluster.Host.t/0`) - the host the event is related to\n\n\n\n\n### `[:xandra, :cluster, :control_connection, :connected]`\n\n\n\nEmitted when the control connection for the cluster is established.\n\n\n\n**Measurements**: *none*\n\n\n\n**Metadata**:\n\n* `:cluster_pid` (`t:pid/0`) - the PID of the cluster process\n* `:cluster_name` (`t:String.t/0` or `nil`) - the name of the cluster executing the event, if provided\nthrough the `:name` option in `Xandra.Cluster.start_link/1`\n* `:host` (`t:Xandra.Cluster.Host.t/0`) - the host the event is related to\n\n\n\n\n### `[:xandra, :cluster, :control_connection, :disconnected | :failed_to_connect]`\n\n\n\nEmitted when the control connection for the cluster disconnects or fails to connect.\n\n\n\n**Measurements**: *none*\n\n\n\n**Metadata**:\n\n* `:cluster_pid` (`t:pid/0`) - the PID of the cluster process\n* `:cluster_name` (`t:String.t/0` or `nil`) - the name of the cluster executing the event, if provided\nthrough the `:name` option in `Xandra.Cluster.start_link/1`\n* `:host` (`t:Xandra.Cluster.Host.t/0`) - the host the event is related to\n* `:reason` (any term) - the reason for the disconnection or failure to connect","ref":"telemetry-events.html#cluster-events"}],"content_type":"text/markdown"}